/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ABSTRACTRESPONSEFILTER_H_
#define ABSTRACTRESPONSEFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "httpresponsemetadata.h"
#include <sigc++/sigc++.h>
#include <cstddef>

class AbstractResponseFilter
{
public:
	typedef SigC::Slot1<AbstractResponseFilter*, HttpResponseMetadata*> NextFilterProvider;
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end) = 0;
	virtual void consumeEOF() = 0;
	
	virtual void destroy() {
		delete this;
	}
	
protected:
	virtual ~AbstractResponseFilter() {}
	
	void* operator new(size_t size) {
		return ::operator new(size);
	}
	
	void operator delete(void* addr) {
		::operator delete(addr);
	}
	/*
	operator delete should be private, but the compiler will then
	force new to be also private. Always use destroy() instead of delete.
	*/
};

#endif

