/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "acceptencoding.h"
#include "stringutils.h"
#include <algorithm>

AcceptEncoding::AcceptEncoding(const HttpHeader& header)
:	m_has_catchall(false),
	m_catchall_quality(0.0),
	m_catchall_quality_explicit(false)
{
	std::list<HttpHeaderElement> elements = header.elements().getElements();
	std::list<HttpHeaderElement>::const_iterator iter = elements.begin();
	for (; iter != elements.end(); ++iter) {
		if (iter->hasParameter("q")) {
			add(iter->getName(), StringUtils::toNumber<float>(iter->getParameterByName("q").getValue()));
		} else {
			add(iter->getName());
		}
	}
}

bool
AcceptEncoding::isAccepted(const std::string& item) const
{
	Quality q = getQuality(item);
	if (!q.isValid())
		return false;
	return (q.getQuality() != 0.0);
}

AcceptEncoding::Quality
AcceptEncoding::getQuality(const std::string& item) const
{
	static const char identity[] = { 'i','d','e','n','t','i','t','y' };
	
	if (item == "*")
		return Quality(m_has_catchall, m_catchall_quality);
	
	bool exists = (m_items.find(item) != m_items.end());
	if (!exists) {
		if (StringUtils::ciEquals(item, identity, identity+sizeof(identity))) {
			return Quality(true, (m_catchall_quality!=0.0 || !m_has_catchall) ? 0.5 : 0.0);
		} else {
			return Quality(m_has_catchall, m_catchall_quality);
		}
	}
	std::map<std::string, float>::const_iterator fit = m_qualities.find(item);
	if (fit == m_qualities.end()) {
		return Quality(true);
	} else {
		return Quality(true, fit->second);
	}
}

void
AcceptEncoding::add(const std::string& item)
{
	if (item == "*") {
		m_has_catchall = true;
		m_catchall_quality = 1.0;
		m_catchall_quality_explicit = false;
	} else {
		m_items.insert(item);
		m_qualities.erase(item);
		m_order.push_back(item);
	}
}

void
AcceptEncoding::add(const std::string& item, float quality)
{
	if (item == "*") {
		m_has_catchall = true;
		m_catchall_quality = quality;
		m_catchall_quality_explicit = true;
	} else {
		m_items.insert(item);
		m_qualities[item] = quality;
		m_order.push_back(item);
	}
}

