/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ACCEPTENCODING_H_
#define ACCEPTENCODING_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include <set>
#include <list>
#include "httpheader.h"

class AcceptEncoding
{
public:
	AcceptEncoding(const HttpHeader& header);
	
	bool isAccepted(const std::string& item) const;
	
	template<typename Iterator>
	Iterator bestAccepted(Iterator begin, Iterator end) const;
private:
	class Quality
	{
	public:
		Quality(bool valid, float qvalue=1.0)
		: m_valid(valid), m_qvalue(qvalue) {}
		
		bool isValid() const { return m_valid; }
		
		float getQuality() const { return m_qvalue; }
	private:
		bool m_valid;
		float m_qvalue;
	};
private:
	void add(const std::string& item);
	
	void add(const std::string& item, float quality);
	
	Quality getQuality(const std::string& item) const;
private:
	std::set<std::string> m_items;
	std::map<std::string, float> m_qualities;
	std::list<std::string> m_order;
	bool m_has_catchall;
	float m_catchall_quality;
	float m_catchall_quality_explicit;
};

template<typename Iterator>
Iterator
AcceptEncoding::bestAccepted(Iterator begin, Iterator end) const
{
	Quality best_quality(false, 0.0);
	Iterator best_option = end;
	for (; begin != end; ++begin) {
		Quality q = getQuality(*begin);
		if (q.isValid() && q.getQuality() != 0.0 && q.getQuality() > best_quality.getQuality()) {
			best_quality = q;
			best_option = begin;
		}
	}
	return best_option;
}

#endif

