/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef BFILTER_H_
#define BFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "clientrequeststream.h"
#include "serverresponsestream.h"
#include "clientinfo.h"
#include "serverinfo.h"
#include "htmlprocessor.h"
#include "httpclientrequest.h"
#include "state.h"
#include "network.h"
#include "intrusiveptr.h"
#include "buffer.h"
#include <string>
#include <deque>
#include <cstddef>

class BFilter
{
	friend class ClientRequestStream;
	friend class ServerResponseStream;
	friend class HTMLProcessor;
public:
	BFilter(Network::Socket csock, State* state);
	void run();
	State& getState() const { return *m_state; }
private:
	typedef IntrusivePtr<HttpClientRequest> RequestPtr;
	typedef std::deque<RequestPtr> RequestQueue;
	void runUnsafe();
	void moreDataForServer(const uint8_t* begin, const uint8_t* end);
	void moreDataForServer(const std::string& data);
	void moreDataForClient(const uint8_t* begin, const uint8_t* end);
	void moreDataForClient(const std::string& data);
	void closeServerConnection(bool servers_initiative=false);
	void switchToServer(const std::string& host, int port);
	void makeServerConnection();
	void flushClientData();
	ClientInfo& clientInfo() { return m_client_info; }
	ServerInfo& serverInfo() { return m_server_info; }
	ClientRequestStream& requestStream() { return m_request_stream; }
	ServerResponseStream& responseStream() { return m_response_stream; }
	RequestQueue& requestsToBeServed() { return m_requestsToBeServed; }
	std::size_t getNumPendingRequests() const { return m_numPendingRequests; }
	void incNumPendingRequests() { ++m_numPendingRequests; }
	void decNumPendingRequests() { --m_numPendingRequests; }
	HtmlProcessor* htmlProcessor() { return &m_html_processor; }
	void appendRequest(HttpClientRequest* request);
	void shiftRequests();
	
	State* m_state;
	Network::Socket m_csock;
	Network::Socket m_ssock;
	std::string m_server_host;
	int m_server_port;
	Buffer m_dataForServer;
	Buffer m_dataForClient;
	HtmlProcessor m_html_processor;
	ServerInfo m_server_info;
	ClientInfo m_client_info;
	ClientRequestStream m_request_stream;
	ServerResponseStream m_response_stream;
	RequestQueue m_requestsToBeServed; // last one may be incomplete
	std::size_t m_numPendingRequests; /* Number of requests that has been sent to
	the server but are not yet (fully) answered. This value is usually 1 or 0
	unless the client does pipelining. */
};

#endif

