/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CONF_H_
#define CONF_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "regexp.h"
#include <string>
#include <cstring>
#include <list>
#include <memory>

class Config
{
	friend class ConfigIO;
public:
	Config();
	Config(const Config& other);
	Config& operator=(const Config& other);
	std::string serialize() const;
	void unserialize(const std::string& data);
	const std::string& getListenAddr() const { return m_listen_addr; }
	void setListenAddr(const std::string& addr) { m_listen_addr = addr; }
	int getListenPort() const { return m_listen_port; }
	void setListenPort(int port) { m_listen_port = port; }
	bool getUseProxy() const { return m_use_proxy; }
	void setUseProxy(bool use_proxy) { m_use_proxy = use_proxy; }
	const std::string& getProxyHost() const { return m_proxy_host; }
	void setProxyHost(const std::string& proxy_host) { m_proxy_host = proxy_host; }
	int getProxyPort() const { return m_proxy_port; }
	void setProxyPort(int proxy_port) { m_proxy_port = proxy_port; }
	const std::list<std::string>& getNoProxyFor() const { return m_no_proxy_for; }
	void setNoProxyFor(const std::list<std::string>& no_proxy_for);
	bool willGoThroughProxy(const std::string& host) const;
	bool getClientCompression() const { return m_client_compression; }
	void setClientCompression(bool on) { m_client_compression = on; }
	bool getDrawBorders() const { return m_draw_borders; }
	void setDrawBorders(bool draw_borders) { m_draw_borders = draw_borders; }
	const unsigned char* getBorderColor() const { return m_border_color; }
	void setBorderColor(const unsigned char *color) { memcpy(m_border_color, color, sizeof(m_border_color)); }
	bool getNoFlash() const { return m_no_flash; }
	bool operator==(const Config& other) const;
private:
	std::string regQuote(const std::string& str);
	
	std::string m_listen_addr;
	int m_listen_port;
	bool m_use_proxy;
	std::string m_proxy_host;
	int m_proxy_port;
	std::list<std::string> m_no_proxy_for;
	std::auto_ptr<RegExp> m_ptrReg_no_proxy_for;
	bool m_client_compression;
	bool m_draw_borders;
	unsigned char m_border_color[3];
	bool m_no_flash;
};

#endif

