/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CONFIGIO_H_
#define CONFIGIO_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "confio.h"
#include "conf.h"
#include <string>
#include <list>
#include <map>

class ConfigIO : public ConfIO
{
public:
	struct Status {
		enum { OK, WARNINGS, ERRORS } status;
		ErrorList errors;
		Status() : status(OK) {}
	};
	ConfigIO() {}
	ConfigIO(const ConfigIO& other) { *this = other; }
	ConfigIO& operator=(const ConfigIO& other);
	void reset();
	Status readConfig(std::istream& strm) { reset(); read(strm); return m_status; }
	void writeConfig(std::ostream& strm) { write(strm, m_elements); }
	const Config& getConfig() const { return m_config; }
	void setConfig(const Config& config);
private:
	typedef std::list<Element>::iterator ElementPointer;
	struct SectionPointer
	{
		ElementPointer findParam(const std::string& key, ElementPointer dflt);
		ElementPointer m_section_element;
		std::list<ElementPointer> m_kv_elements;
	};
	
	virtual bool processElement(const Element& element, int line);
	virtual void processError(const Error& error);
	bool validateSection(const std::string& section, int line);
	bool processKeyValue(const std::string& section, const std::string& key, const std::string& value, int line);
	void assignParam(SectionPointer& sp, const std::string& key, const std::string& value);
	static std::string trim(const std::string& str);
	
	Status m_status;
	Config m_config;
	std::list<Element> m_elements;
	std::string m_cur_section;
	std::map<std::string, SectionPointer> m_sections;
};

#endif

