/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "regexp.h"
#include "util.h"
#include <string>
#include <cstddef>
#include <regex.h>

struct RegExp::Impl
{
	regex_t m_regex;
};

RegExp::RegExp()
:	m_pImpl(0),
	m_flags(0)
{
}

RegExp::RegExp(const std::string& pattern, int flags)
:	m_pattern(pattern),
	m_flags(flags)
{
	m_pImpl = new Impl;
	if (regcomp(&m_pImpl->m_regex, pattern.c_str(), flags)) {
		delete m_pImpl;
		m_pImpl = 0;
	}
}

RegExp::RegExp(const RegExp& other)
{
	if (other.isNull()) {
		RegExp re;
		re.swap(*this);
	} else {
		RegExp re(other.m_pattern, other.m_flags);
		re.swap(*this);
	}
}

RegExp::~RegExp()
{
	if (m_pImpl) {
		regfree(&m_pImpl->m_regex);
		delete m_pImpl;
	}
}

RegExp&
RegExp::operator=(const RegExp& rhs)
{
	RegExp re(rhs);
	re.swap(*this);
	return *this;
}

bool
RegExp::matches(const std::string& str) const
{
	if (isNull())
		return false;
	return !regexec(&m_pImpl->m_regex, str.c_str(), 0, 0, 0);
}

std::string
RegExp::serialize() const
{
	std::ostringstream strm;
	strm << m_pattern.length() << ':' << m_pattern << ':' << m_flags;
	return strm.str();
}

bool
RegExp::unserialize(const std::string& data)
{
	std::istringstream strm(data);
	std::size_t patlen = 0;
	strm >> patlen;
	if (strm.get() != ':')
		return false;
	std::string pattern;
	pattern.resize(patlen);
	strm.read(const_cast<char *>(pattern.data()), patlen);
	pattern.resize(strm.gcount());
	if (strm.get() != ':')
		return false;
	int flags = 0;
	strm >> flags;
	RegExp re(pattern, flags);
	re.swap(*this);
	return true;
}

void
RegExp::swap(RegExp& other)
{
	Impl* impl = other.m_pImpl;
	other.m_pImpl = m_pImpl;
	m_pImpl = impl;
	m_pattern.swap(other.m_pattern);
	int flags = other.m_flags;
	other.m_flags = m_flags;
	m_flags = flags;
}

