/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef RULES_H_
#define RULES_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include "regexp.h"
#include "intrusiveptr.h"

class Rules
{
	friend class RulesReader;
public:
	Rules();
	std::string serialize() const;
	void unserialize(const std::string& data);
	int getFilterParam(const std::string& url) const;
	int getAdParam(const std::string& url) const;
	int getScriptsParam(const std::string& url) const;
	int getJSengineParam(const std::string& url) const;
	int getTargetBlankParam(const std::string& url) const;
private:
	typedef IntrusivePtr<RegExp> RegexPtr;
	class Rule
	{
	public:
		std::string serialize() const;
		void unserialize(const std::string& data);
		int m_filter;
		int m_ad;
		int m_scripts;
		int m_jsengine;
		int m_target_blank;
	};
	struct IntToString
	{
		static std::string convert(int val);
	};
	struct RegexToString
	{
		static std::string convert(const RegexPtr& regex);
	};
	struct StringToInt
	{
		static int convert(const std::string& str);
	};
	struct StringToRegex
	{
		static RegexPtr convert(const std::string& str);
	};
	static std::string serializeRegexMap(const std::map<int, RegexPtr>& m);
	static void unserializeRegexMap(const std::string& data, std::map<int, RegexPtr>& m);
	std::map<int, RegexPtr> m_global_re_by_filter;
	std::map<int, RegexPtr> m_local_re_by_filter;
	std::map<int, RegexPtr> m_global_re_by_ad;
	std::map<int, RegexPtr> m_local_re_by_ad;
	std::map<int, RegexPtr> m_global_re_by_scripts;
	std::map<int, RegexPtr> m_local_re_by_scripts;
	std::map<int, RegexPtr> m_global_re_by_jsengine;
	std::map<int, RegexPtr> m_local_re_by_jsengine;
	std::map<int, RegexPtr> m_global_re_by_target_blank;
	std::map<int, RegexPtr> m_local_re_by_target_blank;
	Rule m_default_rule;
	Rule m_default_global_rule;
	Rule m_default_local_rule;
};

#endif

