/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef RULESREADER_H_
#define RULESREADER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "confio.h"
#include "rules.h"
#include <string>
#include <list>
#include <map>

class RulesReader : public ConfIO
{
public:
	struct Status {
		enum { OK, WARNINGS, ERRORS } status;
		ErrorList errors_global;
		ErrorList errors_local;
		Status() : status(OK) {}
	};
	RulesReader() {}
	void reset();
	Status readRules(std::istream& strm_global, std::istream& strm_local);
	const Rules& getRules() const { return m_rules; }
	void setRules(const Rules& rules) { m_rules = rules; }
private:
	typedef std::map<std::string, std::map<std::string, int> > RuleStorage;
	virtual bool processElement(const Element& element, int line);
	virtual void processError(const Error& error);
	void processSection(const std::string& section, int line);
	void processParameter(const std::string& section, const std::string& key,
	                      const std::string& value, int line);
	int validateParameter(const std::string& key, const std::string& value, int line);
	void processDefaultRule(Rules::Rule& result, const Rules::Rule& base, RuleStorage& rules);
	void groupRules();
	void groupBy(const std::string& key, const std::map<int, std::list<std::string> >& src,
	             std::map<int, Rules::RegexPtr>& dst);
	
	Status m_status;
	Rules m_rules;
	std::string m_cur_section;
	bool m_local;
	RuleStorage m_global_rules;
	RuleStorage m_local_rules;
};

#endif

