/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "debug.h"
#include "ipcportal.h"
#include "ipcmessage.h"
#include <sstream>
#include <string>

Debug::Debug()
:	m_pPortal(0), m_connId(0)
{
}

Debug&
Debug::getInstance()
{
	static Debug instance;
	return instance;
}

void
Debug::setPortal(IPCPortal* portal)
{
	m_pPortal = portal;
	IPCMessage resp;
	m_pPortal->sendRequest(IPCMessage("newConnection"), resp);
	if (resp.getMethod() != "returnConnectionId") {
		m_pPortal = 0;
	} else {
		m_connId = resp.getParameterAsInt(0);
	}
}

void
Debug::signalSessionFinished()
{
	if (m_pPortal) {
		IPCMessage msg("connectionClosed");
		msg.appendParameter(m_connId);
		m_pPortal->sendMessage(msg);
		m_pPortal->flushPendingData();
	}
}

void
Debug::logMessageStarts(const std::string& content, MessageType type, OutgoingResponseType subtype)
{
	if (m_pPortal) {
		IPCMessage msg("logMessageStarts");
		msg.appendParameter(m_connId);
		msg.appendParameter(content);
		msg.appendParameter(messageTypeToString(type));
		msg.appendParameter(messageSubtypeToString(subtype));
		m_pPortal->sendMessage(msg);
		m_pPortal->flushPendingData();
	}
}

void
Debug::logMessageFinishes(MessageType type)
{
	if (m_pPortal) {
		IPCMessage msg("logMessageFinishes");
		msg.appendParameter(m_connId);
		msg.appendParameter(messageTypeToString(type));
		m_pPortal->sendMessage(msg);
		m_pPortal->flushPendingData();
	}
}

void
Debug::logDebug(const std::string& data)
{
	if (m_pPortal) {
		IPCMessage msg("logDebugMessage");
		msg.appendParameter(m_connId);
		msg.appendParameter(data);
		m_pPortal->sendMessage(msg);
		m_pPortal->flushPendingData();
	}
}

void
Debug::logTraffic(const uint8_t* begin, const uint8_t* end, TrafficType ttype)
{
	if (!m_pPortal)
		return;
	
	std::string type;
	switch (ttype) {
	case TRAF_TO_SERVER:
		type = "TO_SERVER";
		break;
	case TRAF_FROM_SERVER:
		type = "FROM_SERVER";
		break;
	case TRAF_TO_CLIENT:
		type = "TO_CLIENT";
		break;
	case TRAF_FROM_CLIENT:
		type = "FROM_CLIENT";
		break;
	default:
		return;
	}
	IPCMessage msg("logTraffic");
	msg.appendParameter(m_connId);
	msg.appendParameter(type);
	msg.appendParameter(begin, end);
	m_pPortal->sendMessage(msg);
	m_pPortal->flushPendingData();
}

void
Debug::logRequestBufferState(int requests_to_be_served, int requests_sent_to_server)
{
	if (m_pPortal) {
		IPCMessage msg("logRequestBufferState");
		msg.appendParameter(m_connId);
		msg.appendParameter(requests_to_be_served);
		msg.appendParameter(requests_sent_to_server);
		m_pPortal->sendMessage(msg);
		m_pPortal->flushPendingData();
	}
}

const std::string&
Debug::messageTypeToString(MessageType type)
{
	static const std::string table[] = {
		"INCOMING_REQUEST",
		"OUTGOING_REQUEST",
		"INCOMING_RESPONSE",
		"OUTGOING_RESPONSE"
	};
	return table[type];
}

const std::string&
Debug::messageSubtypeToString(OutgoingResponseType type)
{
	static const std::string table[] = {
		"", // ORES_NA
		"ORES_NORMAL",
		"ORES_FILTERED",
		"ORES_GENERATED",
		"ORES_ERROR"
	};
	return table[type];
}

