/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUG_H_
#define DEBUG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "ipcportal.h"
#include <sstream>
#include <string>
#include <cstdlib>

#if defined DEBUG && !defined WIN32

#define DEBUGLOG(what) { std::ostringstream strm; strm << what; Debug::getInstance().logDebug(strm.str()); }
#define DEBUGLOG2(what) DEBUGLOG(what << " [" << __FILE__ << ":" << __LINE__ << "]")
#define DBG_ASSERT(what) { if (!(what)) { DEBUGLOG2("Assertion failed: " #what); abort(); } }
#define LOG_TRAFFIC_TO_SERVER(begin, end) Debug::getInstance().logTraffic(begin, end, Debug::TRAF_TO_SERVER)
#define LOG_TRAFFIC_FROM_SERVER(begin, end) Debug::getInstance().logTraffic(begin, end, Debug::TRAF_FROM_SERVER)
#define LOG_TRAFFIC_TO_CLIENT(begin, end) Debug::getInstance().logTraffic(begin, end, Debug::TRAF_TO_CLIENT)
#define LOG_TRAFFIC_FROM_CLIENT(begin, end) Debug::getInstance().logTraffic(begin, end, Debug::TRAF_FROM_CLIENT)

#define LOG_INCOMING_REQUEST_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::INCOMING_REQUEST)
#define LOG_INCOMING_REQUEST_FINISHES() \
Debug::getInstance().logMessageFinishes(Debug::INCOMING_REQUEST)
#define LOG_OUTGOING_REQUEST_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::OUTGOING_REQUEST)
#define LOG_OUTGOING_REQUEST_FINISHES() \
Debug::getInstance().logMessageFinishes(Debug::OUTGOING_REQUEST)
#define LOG_INCOMING_RESPONSE_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::INCOMING_RESPONSE)
#define LOG_INCOMING_RESPONSE_FINISHES() \
Debug::getInstance().logMessageFinishes(Debug::INCOMING_RESPONSE)
#define LOG_OUTGOING_RESPONSE_FINISHES() \
Debug::getInstance().logMessageFinishes(Debug::OUTGOING_RESPONSE)
#define LOG_NORMAL_OUTGOING_RESPONSE_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::OUTGOING_RESPONSE, Debug::ORES_NORMAL)
#define LOG_FILTERED_OUTGOING_RESPONSE_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::OUTGOING_RESPONSE, Debug::ORES_FILTERED)
#define LOG_GENERATED_OUTGOING_RESPONSE(start_line, headers) { \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::OUTGOING_RESPONSE, Debug::ORES_GENERATED); \
LOG_OUTGOING_RESPONSE_FINISHES(); }
#define LOG_ERROR_OUTGOING_RESPONSE(content) { \
Debug::getInstance().logMessageStarts(content, Debug::OUTGOING_RESPONSE, Debug::ORES_ERROR); \
LOG_OUTGOING_RESPONSE_FINISHES(); }
#define LOG_ERROR_OUTGOING_RESPONSE_STARTS(start_line, headers) \
Debug::getInstance().logMessageStarts((start_line)+"\r\n"+(headers), Debug::OUTGOING_RESPONSE, Debug::ORES_ERROR)

#else

#define DEBUGLOG(what)
#define DEBUGLOG2(what)
#define DBG_ASSERT(what)
#define LOG_TRAFFIC_TO_SERVER(begin, end)
#define LOG_TRAFFIC_FROM_SERVER(begin, end)
#define LOG_TRAFFIC_TO_CLIENT(begin, end)
#define LOG_TRAFFIC_FROM_CLIENT(begin, end)

#define LOG_INCOMING_REQUEST_STARTS(start_line, headers)
#define LOG_INCOMING_REQUEST_FINISHES()
#define LOG_OUTGOING_REQUEST_STARTS(start_line, headers)
#define LOG_OUTGOING_REQUEST_FINISHES()
#define LOG_INCOMING_RESPONSE_STARTS(start_line, headers)
#define LOG_INCOMING_RESPONSE_FINISHES()
#define LOG_OUTGOING_RESPONSE_FINISHES()
#define LOG_NORMAL_OUTGOING_RESPONSE_STARTS(start_line, headers)
#define LOG_FILTERED_OUTGOING_RESPONSE_STARTS(start_line, headers)
#define LOG_GENERATED_OUTGOING_RESPONSE(start_line, headers)
#define LOG_ERROR_OUTGOING_RESPONSE(content)
#define LOG_ERROR_OUTGOING_RESPONSE_STARTS(start_line, headers)

#endif

#ifndef WIN32

class Debug
{
public:
	enum TrafficType { TRAF_TO_SERVER, TRAF_FROM_SERVER, TRAF_TO_CLIENT, TRAF_FROM_CLIENT };
	enum MessageType {
		INCOMING_REQUEST,
		OUTGOING_REQUEST,
		INCOMING_RESPONSE,
		OUTGOING_RESPONSE
	};
	enum OutgoingResponseType {
		ORES_NA,
		ORES_NORMAL,
		ORES_FILTERED,
		ORES_GENERATED,
		ORES_ERROR
	};
	static Debug& getInstance();
	void setPortal(IPCPortal* portal);
	void signalSessionFinished();
	void logMessageStarts(const std::string& content, MessageType type, OutgoingResponseType subtype=ORES_NA);
	void logMessageFinishes(MessageType type);
	void logDebug(const std::string& data);
	void logTraffic(const uint8_t* begin, const uint8_t* end, TrafficType ttype);
	void logRequestBufferState(int requests_to_be_served, int requests_sent_to_server);
protected:
	Debug();
private:
	const std::string& messageTypeToString(MessageType type);
	const std::string& messageSubtypeToString(OutgoingResponseType type);
	IPCPortal* m_pPortal;
	int m_connId;
};

#endif // DEBUG

#endif // !WIN32
