/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DECOMPRESSORRESPONSEFILTER_H_
#define DECOMPRESSORRESPONSEFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "abstractresponsefilter.h"
#include "abstractdecompressor.h"
#include <sigc++/sigc++.h>
#include <list>
#include <string>

class DecompressorResponseFilter : public AbstractResponseFilter, public SigC::Object
{
public:
	static AbstractResponseFilter* create(NextFilterProvider* follower,
		HttpResponseMetadata* metadata);
	
	static AbstractResponseFilter* createStack(NextFilterProvider* follower,
		HttpResponseMetadata* metadata, int max_items);
private:
	DecompressorResponseFilter(AbstractResponseFilter* follower, AbstractDecompressor* delegate);
	
	DecompressorResponseFilter(const DecompressorResponseFilter&);            //
	
	DecompressorResponseFilter& operator=(const DecompressorResponseFilter&); // not implemented
public:
	virtual ~DecompressorResponseFilter();
	
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	
	virtual void consumeEOF();
private:
	void processOutputData(const uint8_t* begin, const uint8_t* end);
	
	static NextFilterProvider makeConstructor(NextFilterProvider* follower,
		HttpResponseMetadata* metadata, bool& ok);
		
	static NextFilterProvider makeRecursiveConstructor(NextFilterProvider* follower,
		HttpResponseMetadata* metadata, int max_items, bool& ok);
	
	static AbstractResponseFilter* construct(HttpResponseMetadata* metadata,
		NextFilterProvider* follower, AbstractDecompressor* delegate, const char* encoding);
private:
	AbstractResponseFilter* const m_pFollower;
	AbstractDecompressor* const m_pDelegate;
};

#endif

