/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "deflatedecompressor.h"

DeflateDecompressor::DeflateDecompressor(bool no_zlib_header)
:	m_noZlibHeader(no_zlib_header),
	m_isError(false),
	m_isEOF(false),
	m_isFinished(false)
{
	initStream();
}

DeflateDecompressor::~DeflateDecompressor()
{
	inflateEnd(&m_strm);
}

void
DeflateDecompressor::reset()
{
	inflateReset(&m_strm);
	m_isError = false;
	m_isEOF = false;
	m_isFinished = false;
}

void
DeflateDecompressor::consumeDataChunk(const uint8_t* begin, const uint8_t* end)
{
	if (begin == end) {
		return;
	}
	if (m_isError || m_isEOF || m_isFinished) {
		m_isError = true;
		m_onErrorSignal();
		return;
	}
	m_strm.next_in = const_cast<Bytef*>(begin);
	m_strm.avail_in = end - begin;
	int res;
	do {
		m_strm.next_out = m_outBuf;
		m_strm.avail_out = sizeof(m_outBuf);
		res = inflate(&m_strm, Z_SYNC_FLUSH);
		if (m_strm.next_out != m_outBuf) {
			m_onDataOutputSignal(m_outBuf, m_strm.next_out);
		}
	} while (res == Z_OK && m_strm.avail_out == 0);
	/*
	If inflate() returns Z_OK, it always consumes all of the input,
	unless there is not enough space in the output buffer.
	Now if there is indeed not enough space in the output buffer,
	it may still consume all of the input data. In this case
	(avail_in == 0 && avail_out == 0) we can't say for sure if all the
	consumed input has been processed or not. In this case we have to
	call inflate once more (with avail_in == 0 and avail_out != 0) and be
	prepared to handle Z_BUF_ERROR that will be returned if all of the input
	has been processed on the previous call.
	*/
	if (res == Z_STREAM_END) {
		m_isFinished = true;
	} else if (res != Z_OK && res != Z_BUF_ERROR) {
		m_isError = true;
		m_onErrorSignal();
	}
}

void
DeflateDecompressor::consumeEOF()
{
	if (m_isEOF) {
		return;
	} else {
		m_isEOF = true;
	}
	if (m_isError || !m_isFinished) {
		m_isError = true;
		m_onErrorSignal();
	}
}

void
DeflateDecompressor::initStream()
{
	m_strm.zalloc = Z_NULL;
	m_strm.zfree = Z_NULL;
	m_strm.opaque = Z_NULL;
	m_strm.next_in = Z_NULL;
	if (m_noZlibHeader) {
		inflateInit2(&m_strm, -MAX_WBITS);
	} else {
		inflateInit(&m_strm);
	}
}

