/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEFLATEDECOMPRESSOR_H_
#define DEFLATEDECOMPRESSOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractdecompressor.h"
#include <zlib.h>
#include <sigc++/sigc++.h>

class DeflateDecompressor : public AbstractDecompressor
{
public:
	DeflateDecompressor(bool no_zlib_header);
	virtual ~DeflateDecompressor();
private:
	DeflateDecompressor(const DeflateDecompressor&);            //
	DeflateDecompressor& operator=(const DeflateDecompressor&); // not implemented
public:
	virtual void reset();
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual bool isError() {
		return m_isError;
	}
	virtual SigC::Signal2<void, const uint8_t*, const uint8_t*>& onDataOutput() {
		return m_onDataOutputSignal;
	}
	virtual SigC::Signal0<void>& onError() {
		return m_onErrorSignal;
	}
private:
	void initStream();
private:
	bool m_noZlibHeader;
	bool m_isError;
	bool m_isEOF;
	bool m_isFinished;
	uint8_t m_outBuf[8192];
	z_stream m_strm;
	SigC::Signal2<void, const uint8_t*, const uint8_t*> m_onDataOutputSignal;
	SigC::Signal0<void> m_onErrorSignal;
};

#endif

