/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GIFIMAGEINSPECTOR_H_
#define GIFIMAGEINSPECTOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractimageinspector.h"
#include <vector>
#include <utility>
#include <cstddef>

class GIFImageInspector : public AbstractImageInspector
{
public:
	GIFImageInspector();
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual void reset();
	virtual std::pair<const uint8_t*, const uint8_t*> getAccumulatedData() const;
	virtual bool isInspectionInProgress() const { return m_isInProgress; }
	virtual bool isInspectionFailed() const { return m_isFailed; }
	virtual int getWidth() const { return m_width; }
	virtual int getHeight() const { return m_height; }
	virtual bool isAnimated() const { return m_isAnimated; }
	static const uint8_t SIGNATURE[3];
private:
	enum State {
		ST_SIGNATURE,
		ST_HEADER,
		ST_GLOBAL_COLOR_TABLE,
		ST_BLOCK_BEGIN,
		ST_IMAGE_DESCRIPTOR,
		ST_LOCAL_COLOR_TABLE,
		ST_IMAGE_DATA,
		ST_IMAGE_DATA_SUBBLOCK_OR_TERMINATOR,
		ST_IMAGE_DATA_SUBBLOCK,
		ST_EXTENSION_BLOCK_BEGIN,
		ST_GRAPHIC_CONTROL_EXTENSION_SUBBLOCK_OR_TERMINATOR,
		ST_GRAPHIC_CONTROL_EXTENSION_SUBBLOCK,
		ST_OTHER_EXTENSION_SUBBLOCK_OR_TERMINATOR,
		ST_OTHER_EXTENSION_SUBBLOCK
	};
	enum DelayState {
		DST_DELAY_NOT_ENCOUNTERED,
		DST_WAITING_FOR_GRAPHIC_BLOCK,
		DST_WAITING_FOR_ANOTHER_GRAPHIC_BLOCK
	};
	State m_state;
	DelayState m_delayState;
	std::vector<uint8_t> m_data;
	bool m_isInProgress;
	bool m_isFailed;
	int m_width;
	int m_height;
	bool m_isAnimated;
	std::size_t m_curPos;
	std::size_t m_curBlockSize;
};

#endif

