/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "advancedconfigwindow.h"
#include "application.h"
#include "configio.h"
#include "rulesreader.h"
#include <sstream>
#include <gtkmm/box.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/button.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/frame.h>
#include <gtkmm/notebook.h>
#include <gtkmm/statusbar.h>

AdvancedConfigWindow::AdvancedConfigWindow()
:	m_scrollAtBottom(true),
	m_pConfigBuffer(Gtk::TextBuffer::create()),
	m_pConfigDefaultBuffer(Gtk::TextBuffer::create()),
	m_pRulesBuffer(Gtk::TextBuffer::create()),
	m_pRulesLocalBuffer(Gtk::TextBuffer::create()),
	m_pCurrentBuffer(m_pConfigBuffer),
	m_pLogBuffer(Gtk::TextBuffer::create()),
	m_pWarningTag(Gtk::TextBuffer::Tag::create("warning")),
	m_pErrorTag(Gtk::TextBuffer::Tag::create("error")),
	m_pLogView(manage(new Gtk::TextView(m_pLogBuffer))),
	m_pLineStatusbar(manage(new Gtk::Statusbar)),
	m_pColStatusbar(manage(new Gtk::Statusbar)),
	m_pApplyBtn(manage(new Gtk::Button(Gtk::Stock::APPLY))),
	m_pSaveBtn(manage(new Gtk::Button(Gtk::Stock::SAVE)))
{
	m_pWarningTag->property_foreground() = "#fd687c";
	m_pErrorTag->property_foreground() = "#f7005d";
	m_pLogBuffer->get_tag_table()->add(m_pWarningTag);
	m_pLogBuffer->get_tag_table()->add(m_pErrorTag);
	
	set_title("Advanced Configuration");
	
	Gtk::VBox* vbox = manage(new Gtk::VBox);
	add(*vbox);
	Gtk::Notebook* notebook = manage(new Gtk::Notebook);
	vbox->pack_start(*notebook);
	
	Glib::RefPtr<Gtk::TextBuffer> buffers[4] = {
		m_pConfigBuffer, m_pConfigDefaultBuffer,
		m_pRulesBuffer, m_pRulesLocalBuffer
	};
	Gtk::ScrolledWindow* scrolled_windows[4];
	Gtk::TextView* text_views[4];
	for (int i = 0; i < 4; i++) {
		scrolled_windows[i] = manage(new Gtk::ScrolledWindow);
		scrolled_windows[i]->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		scrolled_windows[i]->set_shadow_type(Gtk::SHADOW_IN);
		scrolled_windows[i]->set_size_request(560, 200);
		text_views[i] = manage(new Gtk::TextView(buffers[i]));
		buffers[i]->signal_modified_changed().connect(SigC::slot(*this, &AdvancedConfigWindow::onModifiedChanged));
		buffers[i]->signal_changed().connect(SigC::slot(*this, &AdvancedConfigWindow::onChanged));
		buffers[i]->signal_mark_set().connect(SigC::slot(*this, &AdvancedConfigWindow::onMarkSet));
		scrolled_windows[i]->add(*text_views[i]);
	}
	text_views[1]->set_editable(false);
	text_views[2]->set_editable(false);
	
	notebook->append_page(*scrolled_windows[0], *manage(new Gtk::Label("config")));
	notebook->append_page(*scrolled_windows[1], *manage(new Gtk::Label("config.default")));
	notebook->append_page(*scrolled_windows[2], *manage(new Gtk::Label("rules")));
	notebook->append_page(*scrolled_windows[3], *manage(new Gtk::Label("rules.local")));
	notebook->signal_switch_page().connect(SigC::slot(*this, &AdvancedConfigWindow::onSwitchTab));
	
	Gtk::Alignment* spacer1 = manage(new Gtk::Alignment);
	spacer1->set_size_request(5, 5);
	vbox->pack_start(*spacer1, Gtk::PACK_SHRINK);
	
	Gtk::HBox* hbox1 = manage(new Gtk::HBox(true));
	vbox->pack_start(*hbox1, Gtk::PACK_SHRINK);
	
	Gtk::HBox* hbox2 = manage(new Gtk::HBox);
	hbox1->pack_start(*hbox2);
	
	Gtk::Alignment* spacer2 = manage(new Gtk::Alignment);
	spacer2->set_size_request(5, 5);
	hbox2->pack_start(*spacer2, Gtk::PACK_SHRINK);
	
	Gtk::Alignment* align1 = manage(new Gtk::Alignment(0.0, 1.0, 0.0, 0.0));
	hbox2->pack_start(*align1);
	
	Gtk::Button* clear_log_btn = manage(new Gtk::Button("Clear Log"));
	clear_log_btn->signal_clicked().connect(SigC::slot(*this, &AdvancedConfigWindow::onClearLog));
	align1->add(*clear_log_btn);
	
	Gtk::Alignment* align2 = manage(new Gtk::Alignment(0.5, 0.5, 0.0, 0.0));
	hbox1->pack_start(*align2);
	hbox1->pack_start(*manage(new Gtk::Alignment));
	
	Gtk::HButtonBox* bbox2 = manage(new Gtk::HButtonBox);
	bbox2->set_spacing(10);
	align2->add(*bbox2);
	
	m_pApplyBtn->set_sensitive(false);
	m_pApplyBtn->signal_clicked().connect(SigC::slot(*this, &AdvancedConfigWindow::onApply));
	bbox2->pack_start(*m_pApplyBtn);
	
	m_pSaveBtn->set_sensitive(false);
	m_pSaveBtn->signal_clicked().connect(SigC::slot(*this, &AdvancedConfigWindow::onSave));
	bbox2->pack_start(*m_pSaveBtn);
	
	Gtk::Frame* log_frame = manage(new Gtk::Frame(" Log "));
	log_frame->set_border_width(1);
	vbox->pack_start(*log_frame);
	
	Gtk::ScrolledWindow* log_scroll_window = manage(new Gtk::ScrolledWindow);
	log_scroll_window->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	log_scroll_window->set_shadow_type(Gtk::SHADOW_IN);
	log_scroll_window->set_size_request(560, 100);
	log_scroll_window->set_border_width(2);
	log_scroll_window->add(*m_pLogView);
	log_frame->add(*log_scroll_window);
	
	Gtk::HBox* statusbar_hbox = manage(new Gtk::HBox);
	vbox->pack_end(*statusbar_hbox, Gtk::PACK_SHRINK);
	
	Gtk::Statusbar* empty_statusbar = manage(new Gtk::Statusbar);
	empty_statusbar->set_has_resize_grip(false);
	statusbar_hbox->pack_start(*empty_statusbar);
	m_pLineStatusbar->set_has_resize_grip(false);
	m_pLineStatusbar->set_size_request(70);
	statusbar_hbox->pack_start(*m_pLineStatusbar, Gtk::PACK_SHRINK);
	m_pColStatusbar->set_size_request(75);
	statusbar_hbox->pack_start(*m_pColStatusbar, Gtk::PACK_SHRINK);
	
	updateCursorPosition();
	
	log_scroll_window->get_vadjustment()->signal_changed().connect(SigC::bind(
		SigC::slot(*this, &AdvancedConfigWindow::onVAdjustmentChanged),
		log_scroll_window->get_vadjustment()));
	log_scroll_window->get_vadjustment()->signal_value_changed().connect(SigC::bind(
		SigC::slot(*this, &AdvancedConfigWindow::onVAdjustmentValueChanged),
		log_scroll_window->get_vadjustment()));
	
	show_all_children();
	
	Log::getInstance().attachListener(this);
	std::list<Log::Entry>::const_iterator iter = Log::getInstance().getEntries().begin();
	for (; iter != Log::getInstance().getEntries().end(); ++iter) {
		append(*iter);
	}
}

AdvancedConfigWindow::~AdvancedConfigWindow()
{
	Log::getInstance().detachListener(this);
}

void
AdvancedConfigWindow::loadConfig(const std::string& data)
{
	m_pConfigBuffer->set_text(data);
}

void
AdvancedConfigWindow::loadConfigDefault(const std::string& data)
{
	m_pConfigDefaultBuffer->set_text(data);
}

void
AdvancedConfigWindow::loadRules(const std::string& data)
{
	m_pRulesBuffer->set_text(data);
}

void
AdvancedConfigWindow::loadRulesLocal(const std::string& data)
{
	m_pRulesLocalBuffer->set_text(data);
}

void
AdvancedConfigWindow::setNotModified()
{
	m_pConfigBuffer->set_modified(false);
	m_pConfigDefaultBuffer->set_modified(false);
	m_pRulesBuffer->set_modified(false);
	m_pRulesLocalBuffer->set_modified(false);
}

void
AdvancedConfigWindow::append(const Log::Entry& entry)
{
	if (m_pLogBuffer->get_char_count() > 0 && entry.isNewLine()) {
		m_pLogBuffer->insert(m_pLogBuffer->end(), "\n");
	}
	switch (entry.getType()) {
		case Log::Entry::T_ERROR:
			m_pLogBuffer->insert_with_tag(m_pLogBuffer->end(), entry.getMessage(), m_pErrorTag);
			break;
		case Log::Entry::T_WARNING:
			m_pLogBuffer->insert_with_tag(m_pLogBuffer->end(), entry.getMessage(), m_pWarningTag);
			break;
		case Log::Entry::T_MESSAGE:
			m_pLogBuffer->insert(m_pLogBuffer->end(), entry.getMessage());
			break;
	}
}

void
AdvancedConfigWindow::clear()
{
	m_pLogBuffer->set_text("");
}

void
AdvancedConfigWindow::onClearLog()
{
	Log::getInstance().clear();
}

void
AdvancedConfigWindow::onSwitchTab(GtkNotebookPage* page, guint page_num)
{
	Glib::RefPtr<Gtk::TextBuffer> buffers[4] = {
		m_pConfigBuffer, m_pConfigDefaultBuffer,
		m_pRulesBuffer, m_pRulesLocalBuffer
	};
	m_pCurrentBuffer = buffers[page_num];
	updateCursorPosition();
}

void
AdvancedConfigWindow::onModifiedChanged()
{
	m_pApplyBtn->set_sensitive(m_pConfigBuffer->get_modified() || m_pRulesLocalBuffer->get_modified());
}

void
AdvancedConfigWindow::onChanged()
{
	updateCursorPosition();
}

void
AdvancedConfigWindow::onMarkSet(const Gtk::TextBuffer::iterator& location, const Glib::RefPtr<Gtk::TextBuffer::Mark>& mark)
{
	updateCursorPosition();
}

void
AdvancedConfigWindow::onApply()
{
	std::string config_source = m_pConfigBuffer->get_text().c_str();
	std::string rules_source = m_pRulesBuffer->get_text().c_str();
	std::string rules_local_source = m_pRulesLocalBuffer->get_text().c_str();
	ConfigIO cio;
	if (!Application::getInstance().processConfig(config_source, cio)) {
		m_pSaveBtn->set_sensitive(false);
		return;
	}
	RulesReader rr;
	if (!Application::getInstance().processRules(rules_source, rules_local_source, rr)) {
		m_pSaveBtn->set_sensitive(false);
		return;
	}
	if (!Application::getInstance().applyNewConfigAndRules(cio, rr.getRules(),
		config_source, rules_source, rules_local_source)) {
		m_pSaveBtn->set_sensitive(false);
		return;
	}
	m_pSaveBtn->set_sensitive(true);
	setNotModified();
}

void
AdvancedConfigWindow::onSave()
{
	bool ok = true;
	if (!Application::getInstance().writeConfig())
		ok = false;
	if (!Application::getInstance().writeLocalRules())
		ok = false;
	m_pSaveBtn->set_sensitive(!ok);
}

void
AdvancedConfigWindow::onVAdjustmentChanged(Gtk::Adjustment* a)
{
	if (m_scrollAtBottom) {
		a->set_value(a->get_upper() - a->get_page_size());
		a->value_changed();
	}
}

void
AdvancedConfigWindow::onVAdjustmentValueChanged(Gtk::Adjustment* a)
{
	m_scrollAtBottom = (a->get_value() >= a->get_upper() - a->get_page_size());
}

void
AdvancedConfigWindow::updateCursorPosition()
{
	Gtk::TextBuffer::iterator pos = m_pCurrentBuffer->get_insert()->get_iter();
	
	std::ostringstream strm1;
	strm1 << (pos.get_line()+1);
	m_pLineStatusbar->pop();
	m_pLineStatusbar->push(Glib::ustring("Line: ")+strm1.str());
	
	std::ostringstream strm2;
	strm2 << (pos.get_line_offset()+1);
	m_pColStatusbar->pop();
	m_pColStatusbar->push(Glib::ustring("Col: ")+strm2.str());
}

