/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef APPLICATION_H_
#define APPLICATION_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <set>
#include <sys/types.h>
#include <gtkmm/main.h>
#include "ipcportal.h"
#include "conf.h"
#include "configio.h"
#include "rulesreader.h"
#include "masterstate.h"
#include "statusdocklet.h"
#include "traymenu.h"
#include "logdialog.h"
#include "basicconfigdialog.h"
#include "advancedconfigwindow.h"
#include "debugwindow.h"

class Application : public SigC::Object
{
	friend int main(int argc, char* argv[]);
private:
	Application(int argc, char* argv[]);
	~Application();
	void run();
public:
	static Application& getInstance() { return *m_spInstance; }
	void showTrayMenu(guint button, guint32 activate_time);
	void showLogDialog();
	void showBasicConfigDialog();
	void showAdvancedConfigWindow();
#ifdef DEBUG
	void showDebugWindow();
#endif
	void quit();
	MasterState& getState() { return m_state; }
	void updateFilteringState();
	bool processConfig(const std::string& config, ConfigIO& cio);
	bool processRules(const std::string& global_rules, const std::string& local_rules, RulesReader& rr);
	bool applyNewConfig(const Config& conf);
	bool applyNewConfigAndRules(const ConfigIO& cio, const Rules& rules,
		const std::string& config_source, const std::string& rules_source, const std::string& rules_local_source);
	bool writeConfig();
	bool writeLocalRules();
protected:
	bool onConnect(Glib::IOCondition evt);
private:
	class BackendLink;
	friend class Application::BackendLink;
	class BackendLink : public SigC::Object
	{
	public:
		BackendLink(Application* app, pid_t pid, int rfd, int wfd);
		~BackendLink();
		pid_t getPid() { return m_pid; }
		void updateFilteringState();
		void updateConfig();
		void updateRules();
	protected:
		bool onIncomingData(Glib::IOCondition evt);
		bool onOutgoingData(Glib::IOCondition evt);
		bool onErrorOrHUP(Glib::IOCondition evt);
	private:
		void terminate();
		void handleBackendMessages();
		void updateWriteExpectation();
		Application* m_pApp;
		pid_t m_pid;
		IPCPortal m_portal;
		bool m_waitingForWrite;
	};
	void addBackend(pid_t pid, int rfd, int wfd);
	void removeBackend(BackendLink* backend);
	void prepareToLaunchBackend(int csock);
	bool checkBackend();
	bool checkConfDir();
	bool readConfig();
	bool readConfigDefault();
	bool readGlobalRules();
	bool readLocalRules();
	static int bindTo(const std::string& addr, int port);
	
	static Application* m_spInstance;
	Gtk::Main m_gtkMain;
	std::string m_confDir;
	std::string m_backendExecutable;
	int m_serverSocket;
	std::set<BackendLink*> m_backends;
	SigC::Connection m_pServerSocketConnection;
	std::string m_config;
	std::string m_configDefault;
	std::string m_globalRules;
	std::string m_localRules;
	ConfigIO m_configIO;
	RulesReader m_rulesReader;
	MasterState m_state;
	StatusDocklet m_statusDocklet;
	TrayMenu m_trayMenu;
	LogDialog m_logDialog;
	BasicConfigDialog m_basicConfigDialog;
	AdvancedConfigWindow m_advConfigWindow;
#ifdef DEBUG
	DebugWindow m_debugWindow;
#endif
};

#endif

