/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "basicconfigdialog.h"
#include "application.h"
#include <sstream>
#include <gtkmm/box.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/button.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/frame.h>
#include <gtkmm/stock.h>
#include <gtkmm/messagedialog.h>

BasicConfigDialog::BasicConfigDialog()
{
	set_title("Basic Config");
	set_border_width(5);
	
	Gtk::VBox* vbox = manage(new Gtk::VBox);
	add(*vbox);
	
	Gtk::Alignment* align1 = manage(new Gtk::Alignment(0.5, 0.5, 0.0, 0.0));
	Gtk::HBox* hbox1 = manage(new Gtk::HBox);
	align1->add(*hbox1);
	vbox->pack_start(*align1);
	
	hbox1->set_border_width(5);
	hbox1->set_spacing(10);
	hbox1->pack_start(*manage(new Gtk::Label("Listen Port")));
	m_pListenPort = manage(new Gtk::Entry);
	m_pListenPort->set_size_request(50);
	hbox1->pack_start(*m_pListenPort);
	
	Gtk::Frame* fwd_frame = manage(new Gtk::Frame(" Forwarding "));
	Gtk::VBox* vbox2 = manage(new Gtk::VBox);
	vbox2->set_border_width(5);
	vbox2->set_spacing(2);
	fwd_frame->add(*vbox2);
	vbox->pack_start(*fwd_frame);
	
	m_pForward = manage(new Gtk::CheckButton("Forward to another proxy"));
	vbox2->pack_start(*m_pForward);
	Gtk::HBox* hbox2 = manage(new Gtk::HBox);
	hbox2->set_spacing(5);
	vbox2->pack_start(*hbox2, Gtk::PACK_SHRINK);
	
	m_pProxyHost = manage(new Gtk::Entry);
	m_pProxyHost->set_size_request(130);
	hbox2->pack_start(*m_pProxyHost);
	hbox2->pack_start(*manage(new Gtk::Label(":")), Gtk::PACK_SHRINK);
	m_pProxyPort = manage(new Gtk::Entry);
	m_pProxyPort->set_size_request(50);
	hbox2->pack_start(*m_pProxyPort);
	
	Gtk::Alignment* spacer1 = manage(new Gtk::Alignment);
	spacer1->set_size_request(5, 5);
	vbox->pack_start(*spacer1, Gtk::PACK_SHRINK);
	
	Gtk::Alignment* bbox_alignment = manage(new Gtk::Alignment(0.5, 0.5, 0.0, 0.0));
	vbox->pack_start(*bbox_alignment, Gtk::PACK_SHRINK);
	
	Gtk::HButtonBox* button_box = manage(new Gtk::HButtonBox);
	button_box->set_spacing(10);
	bbox_alignment->add(*button_box);
	
	Gtk::Button *ok_btn = manage(new Gtk::Button(Gtk::Stock::OK));
	ok_btn->signal_clicked().connect(SigC::slot(*this, &BasicConfigDialog::onApply));
	button_box->pack_start(*ok_btn, Gtk::PACK_SHRINK);
	
	Gtk::Button *cancel_btn = manage(new Gtk::Button(Gtk::Stock::CANCEL));
	cancel_btn->signal_clicked().connect(SigC::slot(*this, &BasicConfigDialog::hide));
	button_box->pack_start(*cancel_btn, Gtk::PACK_SHRINK);
	
	cancel_btn->grab_focus();
	
	show_all_children();
}

void
BasicConfigDialog::loadConfig(const Config& conf)
{
	std::ostringstream strm1;
	strm1 << conf.getListenPort();
	m_pListenPort->set_text(strm1.str());
	m_pForward->set_active(conf.getUseProxy());
	m_pProxyHost->set_text(conf.getProxyHost());
	std::ostringstream strm2;
	strm2 << conf.getProxyPort();
	m_pProxyPort->set_text(conf.getProxyPort() <= 0 ? std::string() : strm2.str());
}

void
BasicConfigDialog::onApply()
{
	if (!validate())
		return;
	
	Config conf(Application::getInstance().getState().getConfig());
	saveToConfig(conf);
	if (Application::getInstance().applyNewConfig(conf)) {
		if (!Application::getInstance().writeConfig()) {
			Application::getInstance().showLogDialog();
		}
	} else {
		Application::getInstance().showLogDialog();
	}
	hide();
}

bool
BasicConfigDialog::validate()
{
	bool forward = m_pForward->get_active();
	int listen_port = -1;
	std::istringstream strm1(m_pListenPort->get_text().c_str());
	strm1 >> listen_port;
	if (listen_port <= 0 || listen_port > 65535) {
		m_pListenPort->grab_focus();
		Gtk::MessageDialog d("Listen port must be an integer in rage of [1, 65535]",
			Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
		d.run();
		return false;
	}
	if (!forward)
		return true;
	
	if (m_pProxyHost->get_text().find_first_not_of(" \t\r\n") == Glib::ustring::npos) {
		m_pProxyHost->grab_focus();
		Gtk::MessageDialog d("Proxy host is not given", Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
		d.run();
		return false;
	}
	int proxy_port = -1;
	std::istringstream strm2(m_pProxyPort->get_text().c_str());
	strm2 >> proxy_port;
	if (proxy_port <= 0 || proxy_port > 65535) {
		m_pProxyPort->grab_focus();
		Gtk::MessageDialog d("Proxy port must be an integer in rage of [1, 65535]",
			Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
		d.run();
		return false;
	}
	return true;
}

void trim_ustring(Glib::ustring& str)
{
	str.erase(0, str.find_first_not_of(" \t\r\n"));
	Glib::ustring::size_type pos = str.find_last_not_of(" \t\r\n");
	if (pos != Glib::ustring::npos) {
		str.erase(pos+1);
	}
}

void
BasicConfigDialog::saveToConfig(Config& conf)
{
	int listen_port = -1;
	std::istringstream strm1(m_pListenPort->get_text().c_str());
	strm1 >> listen_port;
	conf.setListenPort(listen_port);
	conf.setUseProxy(m_pForward->get_active());
	Glib::ustring proxy_host = m_pProxyHost->get_text();
	trim_ustring(proxy_host);
	conf.setProxyHost(proxy_host);
	int proxy_port = -1;
	std::istringstream strm2(m_pProxyPort->get_text().c_str());
	strm2 >> proxy_port;
	conf.setProxyPort(proxy_port);
}

