/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUGCONNECTION_H_
#define DEBUGCONNECTION_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "debugmessage.h"
#include <string>
#include <cstddef>
#include <deque>
#include <gtkmm/eventbox.h>
#include <gtkmm/fixed.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/window.h>
#include <gtkmm/textbuffer.h>
#include <gtkmm/button.h>
#include <gtkmm/label.h>
#include <gtkmm/combo.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/fileselection.h>

class DebugLayout;

class DebugConnection : public Gtk::EventBox
{
public:
	enum TrafficType {
		TRAF_FROM_CLIENT,
		TRAF_TO_CLIENT,
		TRAF_FROM_SERVER,
		TRAF_TO_SERVER
	};
	DebugConnection(int id, DebugLayout* layout, int top);
	virtual ~DebugConnection();
private:
	DebugConnection(const DebugConnection&);            //
	DebugConnection& operator=(const DebugConnection&); // not implemented
public:
	int getId() const { return m_id; }
	int getTop() const { return m_top; }
	void setTop(int top) { m_top = top; }
	int getHeight() const { return m_height; }
	int getBottom() const { return m_top+m_height; }
	bool isClosed() const { return m_closed; }
	void growBy(int increment);
	void markAsClosed();
	void messageStarts(const std::string& content, DebugMessage::Type type, DebugMessage::Subtype subtype);
	void messageFinishes(DebugMessage::Type type);
	void debugMessage(const std::string& content);
	void logTraffic(TrafficType type, const std::string& content);
public:
	SigC::Signal0<void> destroySignal;
protected:
	bool onButtonPress(GdkEventButton* event);
private:
	class ViewWindow : public Gtk::Window
	{
	public:
		ViewWindow(DebugConnection* debug_conn);
		void appendDebugMessage(const std::string& msg);
		void trafficUpdateAvailable(TrafficType type);
	private:
		void onTrafUpdate(TrafficType type);
		void onSaveAs(TrafficType type);
		void onChangeEncoding(TrafficType type, Gtk::Combo* combo);
		Glib::ustring trafficToUnicode(TrafficType type);
	private:
		DebugConnection* m_pDebugConnection;
		Glib::RefPtr<Gtk::TextBuffer> m_pDebugBuffer;
		Glib::RefPtr<Gtk::TextBuffer> m_pTrafficBuffers[4];
		Gtk::Label* m_pTrafficSizeLabels[4];
		Gtk::Button* m_pUpdateButtons[4];
		std::string m_encoding;
	};
	friend class DebugConnection::ViewWindow;
	
	class SaveTrafficDialog : public Gtk::FileSelection
	{
	public:
		SaveTrafficDialog(DebugConnection* debug_conn, TrafficType type);
	private:
		void onResponse(int response);
	private:
		DebugConnection* m_pDebugConnection;
		TrafficType m_trafType;
		Gtk::CheckButton* m_pBlockBoundriesCB;
	};
	friend class DebugConnection::SaveTrafficDialog;
private:
	void destroyViewWindow();
	static void destroySaveTrafficDialog(SaveTrafficDialog* dlg);
private:
	int m_id;
	DebugLayout* m_pLayout;
	int m_top;
	int m_height;
	bool m_closed;
	DebugMessage* m_pLastIncomingRequest;
	DebugMessage* m_pLastOutgoingRequest;
	DebugMessage* m_pLastIncomingResponse;
	DebugMessage* m_pLastOutgoingResponse;
	Gtk::Fixed* m_pContainer;
	Gtk::DrawingArea* m_pBorders[4];
	std::string m_debug;
	std::deque<std::string> m_traffic[4];
	std::size_t m_totalTrafficSize[4];
	std::size_t m_storedTrafficSize[4];
	ViewWindow* m_pViewWindow;
};

#endif

