/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUGLAYOUT_H_
#define DEBUGLAYOUT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "debugconnection.h"
#include "debugmessage.h"
#include <cstddef>
#include <string>
#include <list>
#include <map>
#include <gtkmm/layout.h>

class DebugLayout : public Gtk::Layout
{
public:
	DebugLayout();
	virtual ~DebugLayout();
private:
	DebugLayout(const DebugLayout&);              //
	DebugLayout& operator=(const DebugLayout&);   // not implemented
public:
	enum {
		LAYOUT_PADDING = 10,
		MESSAGE_BORDER = 2,
		MESSAGE_WIDTH = MESSAGE_BORDER*2+6,
		MESSAGE_INITIAL_HEIGHT = MESSAGE_BORDER*2+6,
		MESSAGE_HOR_SPACING = 6,
		MESSAGE_VER_SPACING = 6,
		CONN_BORDER = 2,
		CONN_PADDING = 7,
		CONN_WIDTH = MESSAGE_WIDTH*4+MESSAGE_HOR_SPACING*3+CONN_PADDING*2+CONN_BORDER*2,
		CONN_HOR_SPACING = 12,
		CONN_VER_SPACING = 10
	};
	int getHeight() const { return m_height; }
	int getConnectionsToTrack() const { return m_connectionsToTrack; }
	void setConnectionsToTrack(int val) { m_connectionsToTrack = val; }
	size_t getTrafficLimit() const { return m_trafficLimit; }
	void setTrafficLimit(size_t val) { m_trafficLimit = val; }
	void growBy(int increment);
	int newConnection();
	void connectionClosed(int conn_id);
	void messageStarts(int conn_id, const std::string& content,
		DebugMessage::Type type, DebugMessage::Subtype subtype=DebugMessage::ORES_NA);
	void messageFinishes(int conn_id, DebugMessage::Type type);
	void debugMessage(int conn_id, const std::string& content);
	void logTraffic(int conn_id, DebugConnection::TrafficType type, const std::string& content);
private:
	class Column : public SigC::Object
	{
	public:
		Column(DebugLayout* layout, int left);
	private:
		Column(const Column&);            //
		Column& operator=(const Column&); // not implemented
	public:
		int getLeft() const { return m_left; }
		int getRight() const { return m_left+CONN_WIDTH; }
		int getTop() const;
		bool isInUse() const;
		bool isEmpty() const { return m_connections.empty(); }
		DebugConnection* createConnection(int id);
		void growBy(int increment);
		void trimTopBy(int value);
	private:
		void removeConnection(std::list<DebugConnection*>::iterator iter);
	private:
		DebugLayout* m_pLayout;
		int m_left;
		std::list<DebugConnection*> m_connections;
	};
private:
	void removeOldConnections(int num2keep);
	void removeConnection(std::map<int, DebugConnection*>::iterator iter);
private:
	int m_height;
	int m_width;
	int m_connectionsToTrack;
	size_t m_trafficLimit;
	std::list<Column*> m_columns;
	std::map<int, DebugConnection*> m_connections;
};

#endif

