/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUGMESSAGE_H_
#define DEBUGMESSAGE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <gtkmm/eventbox.h>
#include <gtkmm/fixed.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/window.h>

class DebugMessage : public Gtk::EventBox
{
public:
	enum Type {
		INCOMING_REQUEST,
		OUTGOING_REQUEST,
		INCOMING_RESPONSE,
		OUTGOING_RESPONSE
	};
	enum Subtype {
		ORES_NA,
		ORES_NORMAL,
		ORES_FILTERED,
		ORES_GENERATED,
		ORES_ERROR
	};
	DebugMessage(int top, const std::string& content, Type type, Subtype subtype);
	virtual ~DebugMessage();
private:
	DebugMessage(const DebugMessage&);            //
	DebugMessage& operator=(const DebugMessage&); // not implemented
public:
	int getTop() const { return m_top; }
	int getHeight() const { return m_height; }
	int getBottom() const { return m_top+m_height; }
	bool isActive() const { return m_active; }
	void finish() { m_active = false; }
	void growBy(int increment);
	void markAsClosed();
	static const Gdk::Color& getColorFor(Type type, Subtype subtype);
protected:
	bool onButtonPress(GdkEventButton* event);
private:
	void showContentWindow();
	void destroyContentWindow();
	
	int m_top;
	std::string m_content;
	int m_height;
	bool m_active;
	Gtk::Fixed* m_pContainer;
	Gtk::DrawingArea* m_pBorders[3];
	Gtk::Window* m_pContentWindow;
};

#endif

