/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUGWINDOW_H_
#define DEBUGWINDOW_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "debuglayout.h"
#include "debugconnection.h"
#include "debugmessage.h"
#include <string>
#include <gtkmm/window.h>
#include <gtkmm/layout.h>
#include <gtkmm/adjustment.h>
#include <gtkmm/spinbutton.h>

class DebugWindow : public Gtk::Window
{
	friend class Application;
private:
	DebugWindow();
	DebugWindow(const DebugWindow&);              //
	DebugWindow& operator=(const DebugWindow&);   // not implemented
public:
	int newConnection();
	void connectionClosed(int conn_id);
	void messageStarts(int conn_id, const std::string& content,
		DebugMessage::Type type, DebugMessage::Subtype subtype=DebugMessage::ORES_NA);
	void messageFinishes(int conn_id, DebugMessage::Type type);
	void debugMessage(int conn_id, const std::string& content);
	void logTraffic(int conn_id, DebugConnection::TrafficType type, const std::string& content);
protected:
	void onVAdjustmentChanged(Gtk::Adjustment* a);
	void onVAdjustmentValueChanged(Gtk::Adjustment* a);
	void onTrackConnectionsChange();
	void onTrafficLimitChange();
private:
	Gtk::Widget* makeLegend();
	static Gtk::Widget* makeMessageSymbol(DebugMessage::Type type, DebugMessage::Subtype);
	static Gtk::Widget* makeConnectionSymbol(int width, int height);
private:
	DebugLayout* m_pLayout;
	Gtk::Adjustment m_connectionsAdjustment;
	Gtk::Adjustment m_trafficLimitAdjustment;
	Gtk::SpinButton* m_pConnections;
	Gtk::SpinButton* m_pTrafficLimit;
	bool m_scrollAtBottom;
};

#endif

