/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "imagefactory.h"
#include <glib.h>

ImageFactory&
ImageFactory::getInstance()
{
	static ImageFactory instance;
	return instance;
}

void
ImageFactory::add(const char* name, const guint8* data, int data_length)
{
	getInstance().m_storage[name] = std::pair<const guint8*, int>(data, data_length);
}

Glib::RefPtr<Gdk::Pixbuf>
ImageFactory::get(const char* fname)
{
	StorageType::iterator fit = m_storage.find(fname);
	if (fit == m_storage.end()) {
		g_warning("Could not find %s in ImageFactory", fname);
		return getDefaultPixbuf();
	}
	return Gdk::Pixbuf::create_from_inline(fit->second.second, fit->second.first, false);
}

Glib::RefPtr<Gdk::Pixbuf>
ImageFactory::getDefaultPixbuf()
{
	static Glib::RefPtr<Gdk::Pixbuf> pixbuf(Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, 1, 1));
	return pixbuf;
}

