/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "log.h"

Log&
Log::getInstance()
{
	static Log instance;
	return instance;
}

void
Log::appendEntry(const Log::Entry& entry)
{
	std::set<Listener *>::iterator iter = m_listeners.begin();
	for (; iter != m_listeners.end(); ++iter) {
		(*iter)->append(entry);
	}
	m_entries.push_back(entry);
}

void
Log::appendMessage(const Glib::ustring& msg, bool newline)
{
	appendEntry(Entry(Entry::T_MESSAGE, msg, newline));
}

void
Log::appendWarning(const Glib::ustring& warning, bool newline)
{
	appendEntry(Entry(Entry::T_WARNING, warning, newline));
}

void
Log::appendError(const Glib::ustring& err, bool newline)
{
	appendEntry(Entry(Entry::T_ERROR, err, newline));
}

void
Log::appendOK()
{
	appendMessage("[OK]", false);
}

void
Log::appendFailed()
{
	appendMessage("[", false);
	appendError("FAILED", false);
	appendMessage("]", false);
}

void
Log::clear()
{
	std::set<Listener *>::iterator iter = m_listeners.begin();
	for (; iter != m_listeners.end(); ++iter) {
		(*iter)->clear();
	}
	m_entries.clear();
}

