/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "application.h"
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <gdkmm.h>
#include <gtkmm/image.h>
#include <gtkmm/window.h>

static void sig_child(int signum)
{
	if (signum != SIGCHLD) {
		return;
	}
	int saved_errno = errno;
	while (waitpid(-1, NULL, WNOHANG) > 0);
	errno = saved_errno;
}

static void setup_signals()
{
	struct sigaction sa;
	
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	
	sa.sa_handler = SIG_IGN;
	sigaction(SIGPIPE, &sa, NULL);
	sigaction(SIGUSR1, &sa, NULL);
	
	sa.sa_handler = &sig_child;
	sa.sa_flags = SA_NOCLDSTOP;
	sigaction(SIGCHLD, &sa, NULL);
}

class TransparentWindow : public Gtk::Window
{
public:
	TransparentWindow();
private:
	static const char** getTransparentXPM();
};

TransparentWindow::TransparentWindow()
{
	set_resizable(false);
	set_decorated(false);
	realize();
	const Glib::RefPtr<const Gdk::Drawable> drawable = get_window();
	const Gdk::Color transparent = Gtk::Widget::get_default_style()->get_bg(Gtk::STATE_NORMAL);
	Glib::RefPtr<Gdk::Bitmap> mask;
	const Glib::RefPtr<Gdk::Pixmap> pixmap =
		Gdk::Pixmap::create_from_xpm(drawable, mask, transparent, getTransparentXPM());
	Gtk::Image* image = new Gtk::Image(pixmap, mask);
	add(*manage(image));
	image->set_size_request(1, 1);
	image->show();
	shape_combine_mask(mask, 0, 0);
}

const char**
TransparentWindow::getTransparentXPM()
{
	static const char* xpm[] = {
		"1 1 1 1",
		" 	c None",
		" "
	};
	return xpm;
}

int main(int argc, char *argv[])
{
	setup_signals();
	Application app(argc, argv);
	
	// We have to show a window, or launch feedback won't work
	{
		TransparentWindow wnd;
		wnd.show_now();
	}
	
	app.run();
	return 0;
}

