/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "statusdocklet.h"
#include "application.h"
#include "imagefactory.h"
#include "eggtrayicon.h"
#include <gdk/gdkproperty.h>
#include <gdk/gdkwindow.h>
#include <X11/Xatom.h>

StatusDocklet::StatusDocklet()
:	m_normalIcon(ImageFactory::getInstance().get("trayicon.png")),
	m_unopIcon(ImageFactory::getInstance().get("trayicon_unop.png")),
	m_warnIcon(ImageFactory::getInstance().get("trayicon_warn.png")),
	m_pCurrentIcon(&m_normalIcon),
	m_isOperational(true),
	m_logHasWarnings(false)
{
	m_pWnd = createDocklet();
	Log::getInstance().attachListener(this);
	std::list<Log::Entry>::const_iterator iter = Log::getInstance().getEntries().begin();
	for (; iter != Log::getInstance().getEntries().end(); ++iter) {
		if (iter->getType() != Log::Entry::T_MESSAGE) {
			m_logHasWarnings = true;
			break;
		}
	}
}

StatusDocklet::~StatusDocklet()
{
	Log::getInstance().detachListener(this);
	delete m_pWnd;
}

void
StatusDocklet::setOperational(bool val)
{
	if (m_isOperational == val)
		return;
	
	m_isOperational = val;
	m_pCurrentIcon = val ? (m_logHasWarnings ? &m_warnIcon : &m_normalIcon) : &m_unopIcon;
	m_pWnd->remove();
	m_pWnd->add(*m_pCurrentIcon);
}

void
StatusDocklet::append(const Log::Entry& entry)
{
	if (entry.getType() != Log::Entry::T_MESSAGE && !m_logHasWarnings) {
		m_logHasWarnings = true;
		if (m_isOperational) {
			m_pCurrentIcon = &m_warnIcon;
			m_pWnd->remove();
			m_pWnd->add(*m_pCurrentIcon);
		}
	}
}

void
StatusDocklet::clear()
{
	if (m_logHasWarnings) {
		m_logHasWarnings = false;
		if (m_isOperational) {
			m_pCurrentIcon = &m_normalIcon;
			m_pWnd->remove();
			m_pWnd->add(*m_pCurrentIcon);
		}
	}
}

bool
StatusDocklet::onTrayIconDelete(GdkEventAny* evt)
{
	delete m_pWnd;
	m_pWnd = createDocklet();
	return true;
}

bool
StatusDocklet::onButtonPress(GdkEventButton* event)
{
	Application::getInstance().showTrayMenu(event->button, event->time);
	return true;
}

void
StatusDocklet::onWindowHide()
{
	Application::getInstance().quit();
}

class XEmbedTrayWindow : public Gtk::Window
{
public:
	XEmbedTrayWindow() : Gtk::Window(GTK_WINDOW(egg_tray_icon_new("BFilter"))) {}
};

Gtk::Window*
StatusDocklet::createDocklet()
{
	Gtk::Window *wnd = new XEmbedTrayWindow;
	gtk_widget_realize(GTK_WIDGET(wnd->gobj()));
	if (EGG_TRAY_ICON(wnd->gobj())->manager_window) {
		/*
		Under KDE (version 3.1.2 at least), when an icon is being removed from the tray,
		our icon gets removed as well. Probably a bug in KDE's XEmbed implementation.
		I workaround it by catching the delete event and recreating the docklet.
		*/
		wnd->signal_delete_event().connect(SigC::slot(*this, &StatusDocklet::onTrayIconDelete));
	} else {
		delete wnd;
		wnd = new Gtk::Window;
		wnd->set_title("BFilter");
		wnd->resize(22, 22);
		wnd->signal_hide().connect(SigC::slot(*this, &StatusDocklet::onWindowHide));
		gtk_widget_realize(GTK_WIDGET(wnd->gobj()));
		GdkWindow *window = GTK_WIDGET(wnd->gobj())->window;
		glong data[1];
		/*
		// For KDE >= 1.0
		// Causes problems with WindowMaker. Commented out because of that.
		GdkAtom kwm_dockwindow_atom = gdk_atom_intern("KWM_DOCKWINDOW", FALSE);
		data[0] = TRUE;
		gdk_property_change(window, kwm_dockwindow_atom, kwm_dockwindow_atom,
			32, GDK_PROP_MODE_REPLACE, (guchar *)&data, 1);
		*/
		// For KDE >= 2.0
		GdkAtom kde_net_system_tray_window_for_atom =
			gdk_atom_intern("_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
		data[0] = 0;
		gdk_property_change(window, kde_net_system_tray_window_for_atom,
			(GdkAtom)XA_WINDOW, 32, GDK_PROP_MODE_REPLACE, (guchar *)&data, 1);
	}
	
	m_normalIcon.show();
	m_unopIcon.show();
	m_warnIcon.show();
	
	wnd->add(*m_pCurrentIcon);
	wnd->signal_button_press_event().connect(SigC::slot(*this, &StatusDocklet::onButtonPress));
	wnd->add_events(Gdk::BUTTON_PRESS_MASK);
	wnd->show();
	
	return wnd;
}

