/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "traymenu.h"
#include "application.h"
#include <gtkmm/stock.h>

TrayMenu::TrayMenu()
{
	set_title("BFilter " VERSION);
	
	Gtk::Menu* config_submenu = manage(new Gtk::Menu);
	config_submenu->items().push_back(Gtk::Menu_Helpers::MenuElem("Basic",
		SigC::slot(*this, &TrayMenu::onBasicConfig)));
	config_submenu->items().push_back(Gtk::Menu_Helpers::MenuElem("Advanced",
		SigC::slot(*this, &TrayMenu::onAdvancedConfig)));
	
	
	items().push_back(Gtk::Menu_Helpers::CheckMenuElem("Filter",
		SigC::slot(*this, &TrayMenu::onFilter)));
	m_pFilteringItem = dynamic_cast<Gtk::CheckMenuItem*>(&*--items().end());
	items().push_back(Gtk::Menu_Helpers::CheckMenuElem("Filter Scripts",
		SigC::slot(*this, &TrayMenu::onFilterScripts)));
	m_pScriptFilteringItem = dynamic_cast<Gtk::CheckMenuItem*>(&*--items().end());
	items().push_back(Gtk::Menu_Helpers::SeparatorElem());
#ifdef DEBUG
	items().push_back(Gtk::Menu_Helpers::MenuElem("Debug",
		SigC::slot(*this, &TrayMenu::onDebug)));
#endif
	items().push_back(Gtk::Menu_Helpers::MenuElem("Configuration", *config_submenu));
	items().push_back(Gtk::Menu_Helpers::MenuElem("Log",
		SigC::slot(*this, &TrayMenu::onLog)));
	items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	items().push_back(Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::QUIT,
		SigC::slot(*this, &TrayMenu::onQuit)));
}

// This method can't be called from the constructor because the
// Application object has not yet finished its construction at that time.
void
TrayMenu::init()
{
	m_pFilteringItem->set_active(Application::getInstance().getState().isFilteringEnabled());
	m_pScriptFilteringItem->set_active(Application::getInstance().getState().isScriptFilteringEnabled());
}

void
TrayMenu::onFilter()
{
	m_pScriptFilteringItem->set_sensitive(m_pFilteringItem->get_active());
	Application::getInstance().getState().setFilteringEnabled(m_pFilteringItem->get_active());
	Application::getInstance().updateFilteringState();
}

void
TrayMenu::onFilterScripts()
{
	Application::getInstance().getState().setScriptFilteringEnabled(m_pScriptFilteringItem->get_active());
	Application::getInstance().updateFilteringState();
}

void
TrayMenu::onBasicConfig()
{
	Application::getInstance().showBasicConfigDialog();
}

void
TrayMenu::onAdvancedConfig()
{
	Application::getInstance().showAdvancedConfigWindow();
}

void
TrayMenu::onLog()
{
	Application::getInstance().showLogDialog();
}

#ifdef DEBUG
void
TrayMenu::onDebug()
{
	Application::getInstance().showDebugWindow();
}
#endif

void
TrayMenu::onQuit()
{
	Application::getInstance().quit();
}

