/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "advancedconfigwindow.h"
#include "application.h"
#include "log.h"
#include <string>
#include <cstring>
#include <sstream>
#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/panel.h>
#include <wx/textctrl.h>
#include <wx/listctrl.h>
#include <conf.h>
#include <configio.h>
#include <rules.h>
#include <rulesreader.h>

BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE(myEVT_CARET_MOVE, 7777)
	DECLARE_EVENT_TYPE(myEVT_TEXT_MOD_STATE_UPDATE, 7778)
END_DECLARE_EVENT_TYPES()
DEFINE_EVENT_TYPE(myEVT_CARET_MOVE)
DEFINE_EVENT_TYPE(myEVT_TEXT_MOD_STATE_UPDATE)

BEGIN_EVENT_TABLE(AdvancedConfigWindow, wxFrame)
	EVT_CLOSE(AdvancedConfigWindow::onWindowClose)
	EVT_NOTEBOOK_PAGE_CHANGED(ID_ADVANCED_CONFIG_NOTEBOOK, AdvancedConfigWindow::onTabChanged)
	EVT_BUTTON(wxID_APPLY, AdvancedConfigWindow::onApply)
	EVT_BUTTON(wxID_SAVE, AdvancedConfigWindow::onSave)
	EVT_BUTTON(ID_LOG_CLEAR, AdvancedConfigWindow::onClearLog)
	EVT_COMMAND(IDD_ADVANCED_CONFIG, myEVT_CARET_MOVE, AdvancedConfigWindow::onCaretMove)
	EVT_COMMAND(IDD_ADVANCED_CONFIG, myEVT_TEXT_MOD_STATE_UPDATE, AdvancedConfigWindow::onTextModStateUpdate)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(AdvancedConfigWindow::TextControl, wxTextCtrl)
	EVT_TEXT(wxID_ANY, AdvancedConfigWindow::TextControl::onTextChange)
	EVT_CHAR(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_LEFT_DOWN(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_LEFT_UP(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_MIDDLE_DOWN(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_MIDDLE_UP(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_RIGHT_DOWN(AdvancedConfigWindow::TextControl::onCaretMove)
	EVT_RIGHT_UP(AdvancedConfigWindow::TextControl::onCaretMove)
END_EVENT_TABLE()

AdvancedConfigWindow::AdvancedConfigWindow(wxWindow* parent)
:	wxFrame(parent, IDD_ADVANCED_CONFIG, "Advanced Configuration")
{
	memset(m_textByTab, 0, sizeof(m_textByTab));
	
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	
	wxPanel *panel = new wxPanel(this, -1);
	wxBoxSizer *panelsizer = new wxBoxSizer(wxVERTICAL);
	
	m_pApplyButton = new wxButton(panel, wxID_APPLY, "Apply");
	m_pApplyButton->Disable();
	m_pSaveButton = new wxButton(panel, wxID_SAVE, "Save");
	m_pSaveButton->Disable();
	
	m_pNotebook = new wxNotebook(panel, ID_ADVANCED_CONFIG_NOTEBOOK);
	wxNotebookSizer* nbs = new wxNotebookSizer(m_pNotebook);
	m_pNotebook->AddPage(createConfigPanel(m_pNotebook), "config");
	m_pNotebook->AddPage(createConfigDefaultPanel(m_pNotebook), "config.default");
	m_pNotebook->AddPage(createRulesPanel(m_pNotebook), "rules");
	m_pNotebook->AddPage(createRulesLocalPanel(m_pNotebook), "rules.local");
	m_textByTab[0] = m_pConfigText;
	m_textByTab[1] = m_pConfigDefaultText;
	m_textByTab[2] = m_pRulesText;
	m_textByTab[3] = m_pRulesLocalText;
	panelsizer->Add(nbs, 1, wxGROW, 0);
	
	wxBoxSizer* buttonsizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* clearlog_sizer = new wxBoxSizer(wxHORIZONTAL);
	clearlog_sizer->Add(new wxButton(panel, ID_LOG_CLEAR, "Clear Log"), 0, wxALIGN_LEFT);
	buttonsizer->Add(clearlog_sizer, 1, wxTOP|wxLEFT|wxRIGHT, 5);
	buttonsizer->Add(m_pApplyButton, 0, wxTOP|wxLEFT|wxRIGHT, 5);
	buttonsizer->Add(m_pSaveButton, 0, wxTOP|wxLEFT|wxRIGHT, 5);
	buttonsizer->Add(1, 1, 1);
	panelsizer->Add(buttonsizer, 0, wxEXPAND);
	
	wxStaticBoxSizer* logsizer = new wxStaticBoxSizer(new wxStaticBox(panel, -1, "Log"), wxVERTICAL);
	m_pLogCtrl = new wxListCtrl(panel, ID_LOG_LIST, wxDefaultPosition,
		wxSize(500,100), wxLC_REPORT|wxLC_NO_HEADER|wxLC_SINGLE_SEL);
	m_pLogCtrl->InsertColumn(0, wxEmptyString);
	logsizer->Add(m_pLogCtrl, 1, wxGROW);
	panelsizer->Add(logsizer, 0, wxEXPAND);
	
	panel->SetAutoLayout(true);
	panel->SetSizer(panelsizer);
	panelsizer->Fit(panel);
	panelsizer->SetSizeHints(panel);
	
	topsizer->Add(panel, 1, wxGROW, 0);
	
	wxStatusBar *sbar = new wxStatusBar(this, -1);
	sbar->SetFieldsCount(3);
	static const int widths[] = {-1, 50, 57};
	sbar->SetStatusWidths(3, widths);
	SetStatusBar(sbar);
	updateLineColInfo();
	
	SetAutoLayout(true);
	SetSizer(topsizer);
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);
	
	Log::getInstance().attachListener(this);
}

AdvancedConfigWindow::~AdvancedConfigWindow()
{
	Log::getInstance().detachListener(this);
}

void
AdvancedConfigWindow::loadConfig(const std::string& data)
{
	/*
	Changing the current selected page is a workaround
	for a bug in wxWindows (or maybe I am doing something wrong).
	Without this code, the contents of text controls which are
	located in hidden notebook pages, will become selected when
	that page becomes active.
	*/
	int selection = m_pNotebook->GetSelection();
	m_pNotebook->SetSelection(0);
	m_pConfigText->SetValue(data.c_str());
	m_pNotebook->SetSelection(selection);
}

void
AdvancedConfigWindow::loadConfigDefault(const std::string& data)
{
	int selection = m_pNotebook->GetSelection();
	m_pNotebook->SetSelection(1);
	m_pConfigDefaultText->SetValue(data.c_str());
	m_pNotebook->SetSelection(selection);
}

void
AdvancedConfigWindow::loadRules(const std::string& data)
{
	int selection = m_pNotebook->GetSelection();
	m_pNotebook->SetSelection(2);
	m_pRulesText->SetValue(data.c_str());
	m_pNotebook->SetSelection(selection);
}

void
AdvancedConfigWindow::loadRulesLocal(const std::string& data)
{
	int selection = m_pNotebook->GetSelection();
	m_pNotebook->SetSelection(3);
	m_pRulesLocalText->SetValue(data.c_str());
	m_pNotebook->SetSelection(selection);
}

void
AdvancedConfigWindow::append(const Log::Entry& entry)
{
	static wxColour error_clr(0xf7, 0x00, 0x50);
	static wxColour warning_clr(0xfd, 0x68, 0x7c);
	long id = m_pLogCtrl->InsertItem(m_pLogCtrl->GetItemCount(), entry.getMessage().c_str());
	if (entry.getType() != Log::Entry::T_MESSAGE) {
		wxListItem item;
		item.SetId(id);
		if (entry.getType() == Log::Entry::T_ERROR) {
			item.SetTextColour(error_clr);
		} else if (entry.getType() == Log::Entry::T_WARNING) {
			item.SetTextColour(warning_clr);
		}
		m_pLogCtrl->SetItem(item);
	}
	m_pLogCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
	m_pLogCtrl->EnsureVisible(id);
}

void
AdvancedConfigWindow::clear()
{
	m_pLogCtrl->DeleteAllItems();
	m_pLogCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
}

wxPanel*
AdvancedConfigWindow::createConfigPanel(wxWindow* parent)
{
	wxPanel *panel = new wxPanel(parent, -1);
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	m_pConfigText = new TextControl(this, panel, ID_CONFIG_TEXT);
	topsizer->Add(m_pConfigText, 1, wxGROW);
	panel->SetAutoLayout(true);
	panel->SetSizer(topsizer);
	topsizer->Fit(panel);
	topsizer->SetSizeHints(panel);
	return panel;
}

wxPanel*
AdvancedConfigWindow::createConfigDefaultPanel(wxWindow* parent)
{
	wxPanel *panel = new wxPanel(parent, -1);
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	m_pConfigDefaultText = new TextControl(this, panel, ID_CONFIG_DEFAULT_TEXT, true);
	topsizer->Add(m_pConfigDefaultText, 1, wxGROW);
	panel->SetAutoLayout(true);
	panel->SetSizer(topsizer);
	topsizer->Fit(panel);
	topsizer->SetSizeHints(panel);
	return panel;
}

wxPanel*
AdvancedConfigWindow::createRulesPanel(wxWindow* parent)
{
	wxPanel *panel = new wxPanel(parent, -1);
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	m_pRulesText = new TextControl(this, panel, ID_RULES_TEXT);
	topsizer->Add(m_pRulesText, 1, wxGROW);
	panel->SetAutoLayout(true);
	panel->SetSizer(topsizer);
	topsizer->Fit(panel);
	topsizer->SetSizeHints(panel);
	return panel;
}

wxPanel*
AdvancedConfigWindow::createRulesLocalPanel(wxWindow* parent)
{
	wxPanel *panel = new wxPanel(parent, -1);
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	m_pRulesLocalText = new TextControl(this, panel, ID_RULES_LOCAL_TEXT);
	topsizer->Add(m_pRulesLocalText, 1, wxGROW);
	panel->SetAutoLayout(true);
	panel->SetSizer(topsizer);
	topsizer->Fit(panel);
	topsizer->SetSizeHints(panel);
	return panel;
}

void
AdvancedConfigWindow::updateLineColInfo()
{
	int selection = m_pNotebook->GetSelection();
	if (selection == -1)
		return;
	TextControl* text = m_textByTab[selection];
	if (!text)
		return;
	long x = 0, y = 0;
	text->PositionToXY(text->GetInsertionPoint(), &x, &y);
	GetStatusBar()->SetStatusText("Line: "+wxString::Format("%ld", y+1), 1);
	GetStatusBar()->SetStatusText("Col: "+wxString::Format("%ld", x+1), 2);
}

void
AdvancedConfigWindow::onWindowClose(wxCloseEvent& evt)
{
	if (evt.CanVeto()) {
		evt.Veto();
		Hide();
	} else {
		Destroy();
	}
}

void
AdvancedConfigWindow::onTabChanged(wxNotebookEvent& evt)
{
	updateLineColInfo();
}

void
AdvancedConfigWindow::onCaretMove(wxCommandEvent& evt)
{
	updateLineColInfo();
}

void
AdvancedConfigWindow::onApply(wxCommandEvent& evt)
{
	std::string config_source = m_pConfigText->GetValue().c_str();
	std::string rules_source = m_pRulesText->GetValue().c_str();
	std::string rules_local_source = m_pRulesLocalText->GetValue().c_str();
	ConfigIO cio;
	if (!wxGetApp().processConfig(config_source, cio)) {
		m_pSaveButton->Disable();
		return;
	}
	RulesReader rr;
	if (!wxGetApp().processRules(rules_source, rules_local_source, rr)) {
		m_pSaveButton->Disable();
		return;
	}
	if (!wxGetApp().applyNewConfigAndRules(cio, rr.getRules(), config_source, rules_source, rules_local_source)) {
		m_pSaveButton->Disable();
		return;
	}
	m_pSaveButton->Enable();
	m_pConfigText->setModified(false);
	m_pRulesText->setModified(false);
	m_pRulesLocalText->setModified(false);
}

void
AdvancedConfigWindow::onSave(wxCommandEvent& evt)
{
	bool ok = true;
	if (!wxGetApp().writeConfig())
		ok = false;
	if (!wxGetApp().writeGlobalRules())
		ok = false;
	if (!wxGetApp().writeLocalRules())
		ok = false;
	m_pSaveButton->Enable(!ok);
}

void
AdvancedConfigWindow::onClearLog(wxCommandEvent& evt)
{
	Log::getInstance().clear();
}

void
AdvancedConfigWindow::onTextModStateUpdate(wxCommandEvent& evt)
{
	m_pApplyButton->Enable(m_pConfigText->isModified()
		|| m_pRulesText->isModified() || m_pRulesLocalText->isModified());
}

AdvancedConfigWindow::TextControl::TextControl(AdvancedConfigWindow* owner, wxWindow* parent, wxWindowID id, bool readonly)
:	wxTextCtrl(parent, id, "", wxDefaultPosition, wxSize(500, 200),
		wxTE_PROCESS_TAB|wxTE_MULTILINE|wxTE_DONTWRAP|(readonly ? wxTE_READONLY : 0)),
	m_pOwner(owner),  m_isModified(false)
{
}

void
AdvancedConfigWindow::TextControl::setModified(bool value)
{
	if (value != m_isModified) {
		m_isModified = value;
		wxCommandEvent new_event(myEVT_TEXT_MOD_STATE_UPDATE, IDD_ADVANCED_CONFIG);
		m_pOwner->AddPendingEvent(new_event);
	}
}

void
AdvancedConfigWindow::TextControl::onTextChange(wxCommandEvent& evt)
{
	setModified(true);
	onCaretMove(evt);
}

void
AdvancedConfigWindow::TextControl::onCaretMove(wxEvent& evt)
{
	wxCommandEvent new_event(myEVT_CARET_MOVE, IDD_ADVANCED_CONFIG);
	m_pOwner->AddPendingEvent(new_event);
	evt.Skip();
}

