/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ADVANCEDCONFIGWINDOW_H_
#define ADVANCEDCONFIGWINDOW_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <wx/wx.h>
#include <wx/frame.h>
#include <wx/panel.h>
#include <wx/textctrl.h>
#include <wx/listctrl.h>
#include <wx/notebook.h>
#include <string>
#include "log.h"

class AdvancedConfigWindow : public wxFrame, protected Log::Listener
{
public:
	AdvancedConfigWindow(wxWindow* parent);
	~AdvancedConfigWindow();
	void loadConfig(const std::string& data);
	void loadConfigDefault(const std::string& data);
	void loadRules(const std::string& data);
	void loadRulesLocal(const std::string& data);
protected:
	virtual void append(const Log::Entry& entry);
	virtual void clear();
private:
	class TextControl : public wxTextCtrl
	{
	public:
		TextControl(AdvancedConfigWindow* owner, wxWindow* parent, wxWindowID id, bool readonly=false);
		bool isModified() const { return m_isModified; }
		void setModified(bool value);
	private:
		void onTextChange(wxCommandEvent& evt);
		void onCaretMove(wxEvent& evt);
		AdvancedConfigWindow* m_pOwner;
		bool m_isModified;
		DECLARE_EVENT_TABLE()
	};
private:
	wxPanel* createConfigPanel(wxWindow* parent);
	wxPanel* createConfigDefaultPanel(wxWindow* parent);
	wxPanel* createRulesPanel(wxWindow* parent);
	wxPanel* createRulesLocalPanel(wxWindow* parent);
	void updateLineColInfo();
	void onWindowClose(wxCloseEvent& evt);
	void onTabChanged(wxNotebookEvent& evt);
	void onCaretMove(wxCommandEvent& evt);
	void onTextModStateUpdate(wxCommandEvent& evt);
	void onApply(wxCommandEvent& evt);
	void onSave(wxCommandEvent& evt);
	void onClearLog(wxCommandEvent& evt);
private:
	wxNotebook* m_pNotebook;
	TextControl* m_pConfigText;
	TextControl* m_pConfigDefaultText;
	TextControl* m_pRulesText;
	TextControl* m_pRulesLocalText;
	TextControl* m_textByTab[4];
	wxButton* m_pApplyButton;
	wxButton* m_pSaveButton;
	wxListCtrl* m_pLogCtrl;
	
	DECLARE_EVENT_TABLE()
};

#endif
