/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef APPLICATION_H_
#define APPLICATION_H_

#include <wx/wx.h>
#include <wx/filename.h>
#include <configio.h>
#include <rulesreader.h>
#include "trayicon.h"
#include "traymenu.h"
#include "basicconfigdialog.h"
#include "advancedconfigwindow.h"
#include "log.h"
#include "socketserver.h"
#include "masterstate.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

class Application : public wxApp, protected SocketServer::Listener
{
public:
	Application();
	virtual ~Application();
	virtual bool OnInit();
	bool processConfig(const std::string& config, ConfigIO& cio);
	bool processRules(const std::string& global_rules, const std::string& local_rules, RulesReader& rr);
	const Config& getConfig() const { return m_configIO.getConfig(); }
	const Rules& getRules() const { return m_rulesReader.getRules(); }
	void showTrayMenu();
	void showBasicConfigDialog();
	void showAdvancedConfigWindow();
	void showLogDialog();
	void setFiltering(bool enabled);
	void setScriptFiltering(bool enabled);
	bool applyNewConfig(const Config& config);
	bool applyNewConfigAndRules(const ConfigIO& cio, const Rules& rules,
		const std::string& config_source, const std::string& rules_source, const std::string& rules_local_source);
	bool writeConfig();
	bool writeGlobalRules();
	bool writeLocalRules();
	void quit();
protected:
	virtual void incomingConnection(SOCKET socket);
private:
	bool locateBackend();
	bool readConfig();
	bool readConfigDefault();
	bool readGlobalRules();
	bool readLocalRules();
	
	SocketServer* m_pSocketServer;
	MasterState m_masterState;
	wxFileName m_appDir;
	wxFileName m_confDir;
	std::string m_backendExecutable;
	std::string m_config;
	std::string m_configDefault;
	std::string m_globalRules;
	std::string m_localRules;
	ConfigIO m_configIO;
	RulesReader m_rulesReader;
	TrayIcon m_trayIcon;
	TrayMenu m_trayMenu;
	BasicConfigDialog* m_pBasicConfig;
	AdvancedConfigWindow *m_pAdvancedConfig;
};

DECLARE_APP(Application)

enum {
	IDM_FILTER = wxID_HIGHEST+1,
	IDM_FILTER_SCRIPTS,
	IDM_CONFIG,
	IDM_CONFIG_BASIC,
	IDM_CONFIG_ADVANCED,
	IDM_LOG,
	IDM_QUIT,
	IDD_BASIC_CONFIG,
	ID_LISTEN_PORT,
	ID_FORWARD_PROXY,
	ID_FORWARD_PROXY_HOST,
	ID_FORWARD_PROXY_PORT,
	IDD_LOG,
	ID_LOG_LIST,
	ID_LOG_CLEAR,
	IDD_ADVANCED_CONFIG,
	ID_ADVANCED_CONFIG_NOTEBOOK,
	ID_CONFIG_TEXT,
	ID_CONFIG_DEFAULT_TEXT,
	ID_RULES_TEXT,
	ID_RULES_LOCAL_TEXT
};

#endif
