/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "basicconfigdialog.h"
#include "application.h"
#include "log.h"
#include "logdialog.h"

BEGIN_EVENT_TABLE(BasicConfigDialog, wxDialog)
	EVT_BUTTON(wxID_OK, BasicConfigDialog::onOK)
END_EVENT_TABLE()

BasicConfigDialog::BasicConfigDialog(wxWindow* parent)
:	wxDialog(parent, IDD_BASIC_CONFIG, "Basic Configuration", wxDefaultPosition)
{
	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
	
	wxBoxSizer *hsz1 = new wxBoxSizer(wxHORIZONTAL);
	hsz1->Add(new wxStaticText(this, -1, "Listen Port"), 0, wxALIGN_CENTRE|wxALL, 5);
	m_pListenPort = new wxTextCtrl(this, ID_LISTEN_PORT, "", wxDefaultPosition, wxSize(45, -1));
	hsz1->Add(m_pListenPort, 0, wxALIGN_CENTRE|wxALL, 5);
	
	sizer->Add(hsz1, 0, wxALIGN_CENTRE|wxALL, 5 );
	
	wxStaticBoxSizer *vsz2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, "Forwarding"), wxVERTICAL);
	m_pForwarding = new wxCheckBox(this, ID_FORWARD_PROXY, "Forward to another proxy");
	vsz2->Add(m_pForwarding, 0, wxALIGN_CENTER_VERTICAL|wxALL, 3);
	
	wxBoxSizer *hsz21 = new wxBoxSizer(wxHORIZONTAL);
	m_pProxyHost = new wxTextCtrl(this, ID_FORWARD_PROXY_HOST, "", wxDefaultPosition, wxSize(120,-1));
	hsz21->Add(m_pProxyHost, 0, wxALIGN_CENTRE|wxALL, 5);
	hsz21->Add(new wxStaticText(this, -1, ":"), 0, wxALIGN_CENTRE);
	m_pProxyPort = new wxTextCtrl(this, ID_FORWARD_PROXY_PORT, "", wxDefaultPosition, wxSize(45,-1));
	hsz21->Add(m_pProxyPort, 0, wxALIGN_CENTRE|wxALL, 5 );
	
	vsz2->Add(hsz21, 0, wxALIGN_CENTRE|wxALL, 2);
	
	sizer->Add(vsz2, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5);
	
	wxBoxSizer *hsz3 = new wxBoxSizer(wxHORIZONTAL);
	hsz3->Add(new wxButton(this, wxID_OK, "OK"), 0, wxALIGN_CENTRE|wxALL, 5);
	wxButton* cancel_btn = new wxButton(this, wxID_CANCEL, "Cancel");
	cancel_btn->SetDefault();
	hsz3->Add(cancel_btn, 0, wxALIGN_CENTRE|wxALL, 5);
	
	sizer->Add(hsz3, 0, wxALIGN_CENTRE|wxALL, 5);
	
	SetAutoLayout(true);
	SetSizer(sizer);
	
	sizer->Fit(this);
	sizer->SetSizeHints(this);
}

void
BasicConfigDialog::setConfig(const Config& config)
{
	m_pListenPort->SetValue(wxString::Format("%d", config.getListenPort()));
	m_pForwarding->SetValue(config.getUseProxy());
	m_pProxyHost->SetValue(config.getProxyHost().c_str());
	if (config.getProxyPort() <= 0) {
		m_pProxyPort->SetValue("");
	} else {
		m_pProxyPort->SetValue(wxString::Format("%d", config.getProxyPort()));
	}
}

BasicConfigDialog::CommitStatus
BasicConfigDialog::commit(Config& config)
{
	bool forward = m_pForwarding->GetValue();
	long listen_port = -1;
	if (!m_pListenPort->GetValue().ToLong(&listen_port) || listen_port < 1 || listen_port > 65535) {
		m_pListenPort->SetFocus();
		wxMessageBox("Listen port must be an integer in rage of [1, 65535]", "Error", wxOK|wxICON_EXCLAMATION, this);
		return VALIDATION_FAILED;
	}
	if (forward && m_pProxyHost->GetValue().Trim().IsEmpty()) {
		m_pProxyHost->SetFocus();
		wxMessageBox("Proxy host is not given", "Error", wxOK|wxICON_EXCLAMATION, this);
		return VALIDATION_FAILED;
	}
	long proxy_port = 0;
	if (forward || !m_pProxyPort->GetValue().Trim().IsEmpty()) {
		if (!m_pProxyPort->GetValue().ToLong(&proxy_port) || proxy_port < 1 || proxy_port > 65535) {
			m_pProxyPort->SetFocus();
			wxMessageBox("Proxy port must be an integer in rage of [1, 65535]", "Error", wxOK|wxICON_EXCLAMATION, this);
			return VALIDATION_FAILED;
		}
	}
	
	bool changed = false;
	if (config.getListenPort() != listen_port) {
		changed = true;
	}
	config.setListenPort(listen_port);
	if (config.getUseProxy() != forward) {
		changed = true;
	}
	config.setUseProxy(forward);
	if (config.getProxyHost() != m_pProxyHost->GetValue().Trim().c_str()) {
		changed = true;
	}
	config.setProxyHost(m_pProxyHost->GetValue().Trim().c_str());
	if (config.getProxyPort() != proxy_port) {
		changed = true;
	}
	config.setProxyPort(proxy_port);
	
	return (changed ? PROCEED : NOT_CHANGED);
}

void
BasicConfigDialog::onOK(wxCommandEvent& evt)
{
	Config conf(wxGetApp().getConfig());
	CommitStatus status = commit(conf);
	/*
	if (status == NOT_CHANGED) {
		Log::getInstance().appendMessage("Applying new config [UNCHANGED]");
		evt.Skip();
		return;
	}
	*/
	if (status == NOT_CHANGED || status == PROCEED) {
		if (wxGetApp().applyNewConfig(conf)) {
			if (!wxGetApp().writeConfig()) {
				LogDialog::show();
			}
			evt.Skip();
		} else {
			LogDialog::show();
		}
	}
}

