/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LOG_H_
#define LOG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include <set>

class Log
{
public:
	class Entry
	{
	public:
		enum Type { T_MESSAGE, T_WARNING, T_ERROR };
		Entry(Type type, const std::string& msg)
		: m_type(type), m_msg(msg) {}
		Type getType() const { return m_type; }
		const std::string& getMessage() const { return m_msg; }
	private:
		Type m_type;
		std::string m_msg;
	};
	
	class Listener
	{
		friend class Log;
	public:
		virtual ~Listener() {}
	protected:
		virtual void append(const Entry& entry) = 0;
		virtual void clear() = 0;
	};
	
	static Log& getInstance();
	void attachListener(Listener* listener) { m_listeners.insert(listener); }
	void detachListener(Listener* listener) { m_listeners.erase(listener); }
	void appendMessage(const std::string& msg);
	void appendWarning(const std::string& warning);
	void appendError(const std::string& err);
	void appendEntry(const Entry& entry);
	void clear();
	const std::list<Entry>& getEntries() { return m_entries; }
private:
	Log() {}
	Log(const Log&);
	~Log() {}
	Log& operator=(const Log&);
	
	std::list<Entry> m_entries;
	std::set<Listener *> m_listeners;
};

#endif
