/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "logdialog.h"
#include "application.h"

BEGIN_EVENT_TABLE(LogDialog, wxDialog)
	EVT_BUTTON(ID_LOG_CLEAR, LogDialog::onClearLog)
END_EVENT_TABLE()

LogDialog* LogDialog::m_spInstance = 0;

void
LogDialog::show()
{
	if (m_spInstance) {
		m_spInstance->Show();
		m_spInstance->Raise();
	} else {
		m_spInstance = new LogDialog(wxGetApp().GetTopWindow());
		m_spInstance->Show();
	}
}

LogDialog::LogDialog(wxWindow *parent)
:	wxDialog(parent, IDD_LOG, "BFilter Log", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
{
	wxFlexGridSizer *sizer = new wxFlexGridSizer(1, 0, 0);
	sizer->AddGrowableCol(0);
	sizer->AddGrowableRow(0);
	
	m_pListCtrl = new wxListCtrl(this, ID_LOG_LIST, wxDefaultPosition,
		wxSize(400,150), wxLC_REPORT|wxLC_NO_HEADER|wxLC_SINGLE_SEL);
	m_pListCtrl->InsertColumn(0, wxEmptyString);
	sizer->Add(m_pListCtrl, 0, wxGROW|wxALL, 5);
	
	wxBoxSizer *hsz2 = new wxBoxSizer(wxHORIZONTAL);
	
	wxButton* ok_btn = new wxButton(this, wxID_OK, "OK");
	ok_btn->SetDefault();
	hsz2->Add(ok_btn, 0, wxALIGN_CENTRE|wxLEFT|wxRIGHT|wxBOTTOM, 5);
	hsz2->Add(new wxButton(this, ID_LOG_CLEAR, "Clear Log"), 0, wxALIGN_CENTRE|wxLEFT|wxRIGHT|wxBOTTOM, 5);
	
	sizer->Add(hsz2, 0, wxALIGN_CENTRE|wxALL, 5);
	
	SetAutoLayout(true);
	SetSizer(sizer);
	
	sizer->Fit(this);
	sizer->SetSizeHints(this);
	
	Log::getInstance().attachListener(this);
	std::list<Log::Entry>::const_iterator iter = Log::getInstance().getEntries().begin();
	for (; iter != Log::getInstance().getEntries().end(); ++iter) {
		append(*iter);
	}
}

LogDialog::~LogDialog()
{
	Log::getInstance().detachListener(this);
	m_spInstance = 0;
}

void
LogDialog::append(const Log::Entry& entry)
{
	static wxColour error_clr(0xf7, 0x00, 0x5d);
	static wxColour warning_clr(0xfd, 0x68, 0x7c);
	long id = m_pListCtrl->InsertItem(m_pListCtrl->GetItemCount(), entry.getMessage().c_str());
	if (entry.getType() != Log::Entry::T_MESSAGE) {
		wxListItem item;
		item.SetId(id);
		if (entry.getType() == Log::Entry::T_ERROR) {
			item.SetTextColour(error_clr);
		} else if (entry.getType() == Log::Entry::T_WARNING) {
			item.SetTextColour(warning_clr);
		}
		m_pListCtrl->SetItem(item);
	}
	m_pListCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
	m_pListCtrl->EnsureVisible(id);
}

void
LogDialog::clear()
{
	m_pListCtrl->DeleteAllItems();
	m_pListCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
}

void
LogDialog::onClearLog(wxCommandEvent& evt)
{
	Log::getInstance().clear();
}

