/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "masterstate.h"
#include <cstddef>
#include <string>

MasterState::MasterState()
{
	SECURITY_ATTRIBUTES sa;
	sa.nLength = sizeof(sa);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = TRUE;
	m_hShmMutex = CreateMutex(&sa, TRUE, NULL);
	m_hShm = CreateFileMapping(INVALID_HANDLE_VALUE, &sa, PAGE_READWRITE, 0, sizeof(SharedMemory), NULL);
	m_pShm = (SharedMemory *)MapViewOfFile(m_hShm, FILE_MAP_WRITE, 0, 0, 0);
	m_pShm->m_configMod = 0;
	m_pShm->m_rulesMod = 0;
	m_pShm->m_filteringEnabled = true;
	m_pShm->m_scriptFilteringEnabled = true;
	m_pShm->m_masterDead = false;
	std::string conf_serialized = m_config.serialize();
	m_pShm->m_hConfigShm = CreateFileMapping(INVALID_HANDLE_VALUE, &sa, PAGE_READWRITE, 0,
	                                         sizeof(std::size_t)+conf_serialized.length(), NULL);
	void *conf_shm = MapViewOfFile(m_pShm->m_hConfigShm, FILE_MAP_WRITE, 0, 0, 0);
	*(std::size_t *)conf_shm = conf_serialized.length();
	conf_serialized.copy(((char *)conf_shm)+sizeof(std::size_t), conf_serialized.length());
	UnmapViewOfFile(conf_shm);
	std::string rules_serialized = m_rules.serialize();
	m_pShm->m_hRulesShm = CreateFileMapping(INVALID_HANDLE_VALUE, &sa, PAGE_READWRITE, 0,
	                                        sizeof(std::size_t)+rules_serialized.length(), NULL);
	void *rules_shm = MapViewOfFile(m_pShm->m_hRulesShm, FILE_MAP_WRITE, 0, 0, 0);
	*(std::size_t *)rules_shm = rules_serialized.length();
	rules_serialized.copy(((char *)rules_shm)+sizeof(std::size_t), rules_serialized.length());
	UnmapViewOfFile(rules_shm);
	ReleaseMutex(m_hShmMutex);
}

MasterState::~MasterState()
{
	m_pShm->m_masterDead = true;
	CloseHandle(m_pShm->m_hConfigShm);
	CloseHandle(m_pShm->m_hRulesShm);
	UnmapViewOfFile(m_pShm);
	CloseHandle(m_hShm);
}

void
MasterState::setConfig(const Config& config)
{
	m_config = config;
	std::string conf_serialized = m_config.serialize();
	WaitForSingleObject(m_hShmMutex, INFINITE);
	CloseHandle(m_pShm->m_hConfigShm);
	m_pShm->m_hConfigShm = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0,
	                                         sizeof(std::size_t)+conf_serialized.length(), NULL);
	void *conf_shm = MapViewOfFile(m_pShm->m_hConfigShm, FILE_MAP_WRITE, 0, 0, 0);
	*(std::size_t *)conf_shm = conf_serialized.length();
	conf_serialized.copy(((char *)conf_shm)+sizeof(std::size_t), conf_serialized.length());
	UnmapViewOfFile(conf_shm);
	ReleaseMutex(m_hShmMutex);
}

void
MasterState::setRules(const Rules& rules)
{
	m_rules = rules;
	std::string rules_serialized = m_rules.serialize();
	WaitForSingleObject(m_hShmMutex, INFINITE);
	CloseHandle(m_pShm->m_hRulesShm);
	m_pShm->m_hRulesShm = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0,
	                                        sizeof(std::size_t)+rules_serialized.length(), NULL);
	void *rules_shm = MapViewOfFile(m_pShm->m_hRulesShm, FILE_MAP_WRITE, 0, 0, 0);
	*(std::size_t *)rules_shm = rules_serialized.length();
	rules_serialized.copy(((char *)rules_shm)+sizeof(std::size_t), rules_serialized.length());
	UnmapViewOfFile(rules_shm);
	ReleaseMutex(m_hShmMutex);
}

