/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GZIPCOMPRESSOR_H_
#define GZIPCOMPRESSOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractcompressor.h"
#include <cstddef>
#include <zlib.h>
#include <sigc++/sigc++.h>

class GzipCompressor : public AbstractCompressor
{
public:
	GzipCompressor();
private:
	GzipCompressor(const GzipCompressor&);            //
	GzipCompressor& operator=(const GzipCompressor&); // not implemented
public:
	virtual void reset();
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual bool isError() {
		return m_isError;
	}
	virtual SigC::Signal2<void, const uint8_t*, const uint8_t*>& onDataOutput() {
		return m_onDataOutputSignal;
	}
	virtual SigC::Signal0<void>& onError() {
		return m_onErrorSignal;
	}
private:
	void initStream();
	void writeFooter(uint8_t* where);
private:
	enum { SYNC_INTERVAL = 4096 };
	bool m_isError;
	bool m_isEOF;
	bool m_isFirstBlock;
	uint32_t m_dataLength;
	uint32_t m_dataCRC32;
	std::size_t m_unsyncedSize;
	uint8_t m_outBuf[10+4096+8]; // header + data + footer
	uint8_t* m_outBufPos;
	z_stream m_strm;
	SigC::Signal2<void, const uint8_t*, const uint8_t*> m_onDataOutputSignal;
	SigC::Signal0<void> m_onErrorSignal;
};

#endif

