/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpclientrequest.h"
#include "debug.h"
#include <string>

HttpClientRequest::HttpClientRequest(const HttpRequestMetadata& metadata)
:	m_metadata(metadata), m_orig_metadata(metadata),
	m_is_complete(false), m_is_already_served(false), m_intercept_handler(IH_NONE)
{
	std::string path = metadata.requestLine().getURI().getRawPath();
	if (path == "bf.intercept-request/image_substitute") {
		m_intercept_handler = IH_IMAGE_SUBSTITUTE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
	} else if (path == "bf.intercept-request/flash_substitute") {
		m_intercept_handler = IH_FLASH_SUBSTITUTE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
	} else if (path == "bf.intercept-request/error_substitute") {
		m_intercept_handler = IH_ERROR_SUBSTITUTE;
	} else if (path == "bf.intercept-request/image_analyze") {
		m_intercept_handler = IH_IMAGE_ANALYZE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
		m_metadata.requestLine().setURI(URI(getInterceptParam("url")));
	} else if (path == "bf.intercept-request/flash_analyze") {
		m_intercept_handler = IH_FLASH_ANALYZE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
		m_metadata.requestLine().setURI(URI(getInterceptParam("url")));
	}
	m_is_closing = (metadata.requestLine().getHttpVersion() < HttpVersion::HTTP_1_1
		|| m_metadata.headers().getHeader("Connection").elements().hasElement("close")
		|| m_metadata.headers().getHeader("Proxy-Connection").elements().hasElement("close"));
}

void
HttpClientRequest::appendBodyData(const std::string& data)
{
	m_body_data += data;
}

void
HttpClientRequest::appendBodyData(const uint8_t* begin, const uint8_t* end)
{
	m_body_data.append(reinterpret_cast<const char*>(begin), reinterpret_cast<const char*>(end));
}

void
HttpClientRequest::setComplete()
{
	m_is_complete = true;
}

bool
HttpClientRequest::isToBeSent() const
{
	InterceptHandler ih = getInterceptHandler();
	return !(ih == IH_IMAGE_SUBSTITUTE || ih == IH_FLASH_SUBSTITUTE || ih == IH_ERROR_SUBSTITUTE);
}

bool
HttpClientRequest::isIdempotent() const
{
	const std::string& method = m_metadata.requestLine().getMethod();
	return (method == "GET" || method == "HEAD" || method == "PUT"
	        || method == "DELETE" || method == "OPTIONS" || method == "TRACE");
}

std::string
HttpClientRequest::getInterceptParam(const std::string& name) const
{
	return getInterceptParam(name, m_intercept_params);
}

std::string
HttpClientRequest::getInterceptParam(const std::string& name, const std::list<NameValuePair>& param_list)
{
	std::list<NameValuePair>::const_iterator iter = param_list.begin();
	for (; iter != param_list.end(); ++iter) {
		if (iter->getName() == name) {
			return iter->getValue();
		}
	}
	return "";
}

