/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPCLIENTREQUEST_H_
#define HTTPCLIENTREQUEST_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "uri.h"
#include "httprequestmetadata.h"
#include "namevaluepair.h"
#include "intrusiveptr.h"
#include "types.h"
#include <string>
#include <list>

class HttpClientRequest : public IntrusiveRefCounter
{
public:
	enum InterceptHandler {
		IH_NONE,
		IH_IMAGE_SUBSTITUTE,
		IH_FLASH_SUBSTITUTE,
		IH_IMAGE_ANALYZE,
		IH_FLASH_ANALYZE,
		IH_ERROR_SUBSTITUTE
	};
	
	HttpClientRequest(const HttpRequestMetadata& metadata);
	
	HttpRequestMetadata& metadata() { return m_metadata; }
	
	const HttpRequestMetadata& metadata() const { return m_metadata; }
	
	const HttpRequestMetadata& getOrigMetadata() const { return m_orig_metadata; }
	
	const std::string& getBodyData() const { return m_body_data; }
	
	void appendBodyData(const std::string& data);
	
	void appendBodyData(const uint8_t* begin, const uint8_t* end);
	
	bool isComplete() const { return m_is_complete; }
	
	void setComplete();
	
	bool isAlreadyServed() const { return m_is_already_served; }
	
	void setAlreadyServed() { m_is_already_served = true; }
	
	bool isClosingRequest() const { return m_is_closing; }
	
	bool isToBeSent() const;
	
	bool isIdempotent() const;
	
	InterceptHandler getInterceptHandler() const { return m_intercept_handler; }
	
	void setInterceptHandler(InterceptHandler ih) { m_intercept_handler = ih; }
	
	const std::list<NameValuePair>& getInterceptParams() const { return m_intercept_params; }
	
	std::string getInterceptParam(const std::string& name) const;
	
	static std::string getInterceptParam(const std::string& name, const std::list<NameValuePair>& param_list);
	
	const std::string& getErrorResponse() const { return m_error_response; }
	
	void setErrorResponse(const std::string& error_response) { m_error_response = error_response; }
private:
	HttpRequestMetadata m_metadata;
	HttpRequestMetadata m_orig_metadata;
	std::string m_body_data; // possibly incomplete
	bool m_is_closing;
	bool m_is_complete;
	bool m_is_already_served;
	InterceptHandler m_intercept_handler;
	std::list<NameValuePair> m_intercept_params;
	std::string m_error_response;
};

#endif

