/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPERRORMESSAGES_H_
#define HTTPERRORMESSAGES_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httprequestline.h"
#include "httpstatusline.h"

class HttpErrorMessages
{
public:
	static std::string errMalformedRequestLine(const HttpRequestLine& request_line);
	
	static std::string errBadRequestURI(const HttpRequestLine& request_line, const std::string& details);
	
	static std::string errUnsupportedRequestMethod(const HttpRequestLine& request_line);
	
	static std::string errUnsupportedExpectation(const HttpRequestLine& request_line, const std::string& expectation);
	
	static std::string errMalformedRequestHeader(const HttpRequestLine& request_line);
	
	static std::string errMalformedResponseHeader(const HttpRequestLine& request_line);
	
	static std::string errRequestBodyLengthRequired(const HttpRequestLine& request_line);
	
	static std::string errMalformedChunkInRequest(const HttpRequestLine& request_line);
	
	static std::string errMalformedChunkInResponse(const HttpRequestLine& request_line);
	
	static std::string errUnexpectedServerEOF(const HttpRequestLine& request_line);
	
	static std::string errDNS(const HttpRequestLine& request_line, const std::string& host);
	
	static std::string errConnRefused(const HttpRequestLine& request_line, const std::string& host, int port);
	
	static std::string errConnTimeout(const HttpRequestLine& request_line, const std::string& host, int port);
	
	static std::string errNetworkUnreachable(const HttpRequestLine& request_line, const std::string& host, int port);
	
	static std::string errHostUnreachable(const HttpRequestLine& request_line, const std::string& host, int port);
	
	static std::string errInternal(const HttpRequestLine& request_line, const std::string& details);
private:
	static std::string formatMessage(const HttpRequestLine& request_line,
		const HttpStatusLine& status, const std::string& error_text);
	
	static const char *m_html_error_header;
};

#endif
