/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPHEADERELEMENT_H_
#define HTTPHEADERELEMENT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "namevaluepair.h"
#include <string>
#include <list>

class HttpHeaderElement : public NameValuePair
{
public:
	HttpHeaderElement(const std::string& name, const std::string& value="")
	: NameValuePair(name, value) {}
	
	HttpHeaderElement(const std::string& name, const std::string& value, const std::list<NameValuePair>& params)
	: NameValuePair(name, value), m_params(params) {}
	
	std::list<NameValuePair>& parameters() { return m_params; }
	
	const std::list<NameValuePair>& parameters() const { return m_params; }
	
	bool hasParameter(const std::string& pname) const;
	
	NameValuePair getParameterByName(const std::string& pname) const;
	
	std::list<NameValuePair> getParametersByName(const std::string& pname) const;
	
	std::string toString() const;
	
	static std::string valueToString(const std::string& value);
	
	static std::string quoteValue(const std::string& value);
private:
	static bool valueRequiresQuoting(const std::string& value);
	
	static const char *prepareCharSafetyTable();
	
	static const char *prepareCharEscapeTable();
private:
	static const char *m_char_safety_table;
	static const char *m_char_escape_table;
	std::list<NameValuePair> m_params;
};

#endif

