/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpheaderscollection.h"
#include "httpheader.h"
#include "stringutils.h"
#include <string>
#include <map>
#include <sstream>

void
HttpHeadersCollection::addHeader(const std::string& name, const std::string& value)
{
	std::string lcname = StringUtils::toLowerStr(name);
	HeadersType::iterator fit = m_headers.find(lcname);
	if (fit != m_headers.end()) {
		fit->second.appendValue(value);
	} else {
		m_headers.insert(HeadersType::value_type(lcname, HttpHeader(name, value)));
	}
}

void
HttpHeadersCollection::setHeader(const HttpHeader& header)
{
	HeadersType::iterator fit = m_headers.find(header.getLowercaseName());
	if (fit != m_headers.end()) {
		fit->second = header;
	} else {
		m_headers.insert(HeadersType::value_type(header.getLowercaseName(), header));
	}
}

HttpHeader
HttpHeadersCollection::getHeader(const std::string& name) const
{
	HeadersType::const_iterator fit = m_headers.find(StringUtils::toLowerStr(name));
	if (fit == m_headers.end()) {
		return HttpHeader("", "");
	}
	return fit->second;
}

HttpHeader*
HttpHeadersCollection::getHeaderPtr(const std::string& name)
{
	HeadersType::iterator fit = m_headers.find(StringUtils::toLowerStr(name));
	if (fit == m_headers.end()) {
		return 0;
	}
	return &fit->second;
}

const HttpHeader*
HttpHeadersCollection::getHeaderPtr(const std::string& name) const
{
	HeadersType::const_iterator fit = m_headers.find(StringUtils::toLowerStr(name));
	if (fit == m_headers.end()) {
		return 0;
	}
	return &fit->second;
}

void
HttpHeadersCollection::removeHeader(const std::string& name)
{
	m_headers.erase(StringUtils::toLowerStr(name));
}

bool
HttpHeadersCollection::hasHeader(const std::string& name) const
{
	return (m_headers.find(StringUtils::toLowerStr(name)) != m_headers.end());
}

std::string
HttpHeadersCollection::toString() const
{
	std::ostringstream strm;
	toStream(strm);
	return strm.str();
	
}

void
HttpHeadersCollection::toStream(std::ostream& strm) const
{
	HeadersType::const_iterator iter = m_headers.begin();
	for (; iter != m_headers.end(); ++iter) {
		strm << iter->second;
	}
}

