/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPHEADERSCOLLECTION_H_
#define HTTPHEADERSCOLLECTION_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpheader.h"
#include <string>
#include <map>
#include <iosfwd>

class HttpHeadersCollection
{
public:
	void addHeader(const std::string& name, const std::string& value);
	
	void setHeader(const HttpHeader& header);
	
	HttpHeader getHeader(const std::string& name) const;
	
	HttpHeader *getHeaderPtr(const std::string& name);
	
	const HttpHeader* getHeaderPtr(const std::string& name) const;
	
	void removeHeader(const std::string& name);
	
	bool hasHeader(const std::string& name) const;
	
	void clear() { m_headers.clear(); }
	
	std::string toString() const;
	
	void toStream(std::ostream& strm) const;
private:
	typedef std::map<std::string, HttpHeader> HeadersType;
	HeadersType m_headers;
};

inline std::ostream&
operator<<(std::ostream& strm, const HttpHeadersCollection& collection)
{
	collection.toStream(strm);
	return strm;
}

#endif

