/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httprequest.h"
#include "conf.h"
#include "stringutils.h"

HttpRequest::HttpRequest(const std::string& method, const URI& uri, const std::string& body)
:	m_request_line(method, uri, HttpVersion::HTTP_1_1), m_body(body)
{}

std::string
HttpRequest::toString() const
{
	return toString(false);
}

std::string
HttpRequest::toString(bool through_proxy) const
{
	HttpHeadersCollection headers(m_headers);
	std::string host = getURI().getHost();
	int port = getURI().getPort();
	if (port >= 0) {
		host += ":"+StringUtils::fromNumber(port);
	}
	headers.setHeader(HttpHeader("Host", host));
	headers.setHeader(HttpHeader("Content-Length", StringUtils::fromNumber(m_body.length())));
	headers.setHeader(HttpHeader("Connection", "close"));
	headers.removeHeader("Expect");
	return m_request_line.toString(!through_proxy)+"\r\n"+headers.toString()+"\r\n"+body();
}

