/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPREQUEST_H_
#define HTTPREQUEST_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpmessage.h"
#include "httprequestline.h"
#include "uri.h"

/*
 * This class is only meant to be used with HttpFetcher
 */
class HttpRequest : public HttpMessage
{
public:
	HttpRequest(const std::string& method, const URI& uri, const std::string& body="");
	
	virtual const HttpStartLine& getStartLine() const { return m_request_line; }
	
	virtual HttpHeadersCollection& headers() { return m_headers; }
	
	virtual const HttpHeadersCollection& headers() const { return m_headers; }
	
	virtual std::string& body() { return m_body; }
	
	virtual const std::string& body() const { return m_body; }
	
	virtual const std::string& getMethod() const { return m_request_line.getMethod(); }
	
	virtual void setMethod(const std::string& method) { m_request_line.setMethod(method); }
	
	virtual const URI& getURI() const { return m_request_line.getURI(); }
	
	virtual void setURI(const URI& uri) { m_request_line.setURI(uri); }
	
	virtual std::string toString() const;
	
	virtual std::string toString(bool through_proxy) const;
private:
	HttpRequestLine m_request_line;
	HttpHeadersCollection m_headers;
	std::string m_body;
};

#endif

