/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httprequestline.h"
#include <sstream>

HttpRequestLine::HttpRequestLine(const std::string& method, const URI& uri, const HttpVersion& http_version)
:	m_method(method),
	m_uri(uri),
	m_httpVersion(http_version)
{
}

std::string
HttpRequestLine::toString() const
{
	std::ostringstream strm;
	toStream(strm, false);
	return strm.str();
}

void
HttpRequestLine::toStream(std::ostream& strm) const
{
	toStream(strm, false);
}

std::string
HttpRequestLine::toString(bool relative_url) const
{
	std::ostringstream strm;
	toStream(strm, relative_url);
	return strm.str();
}

void
HttpRequestLine::toStream(std::ostream& strm, bool relative_url) const
{
	strm << m_method << ' ';
	if (relative_url && m_uri.isAbsolute()) {
		URI uri(m_uri);
		uri.makeRelative();
		strm << uri;
	} else {
		strm << m_uri;
	}
	strm << ' ' << m_httpVersion;
}

