/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPRESPONSEMETADATA_H_
#define HTTPRESPONSEMETADATA_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpmessagemetadata.h"
#include "httpstatusline.h"
#include "httpheaderscollection.h"

class HttpResponseMetadata : public HttpMessageMetadata
{
public:
	HttpResponseMetadata(const HttpStatusLine& status_line, const HttpHeadersCollection& headers)
	: m_status_line(status_line), m_headers(headers), m_body_status(BODY_SIZE_UNKNOWN), m_body_size(0) {}
	
	HttpResponseMetadata(const HttpStatusLine& status_line)
	: m_status_line(status_line), m_body_status(BODY_SIZE_UNKNOWN), m_body_size(0) {}
	
	virtual const HttpStartLine& getStartLine() const { return m_status_line; }
	
	HttpStatusLine& statusLine() { return m_status_line; }
	
	const HttpStatusLine& statusLine() const { return m_status_line; }
	
	virtual HttpHeadersCollection& headers() { return m_headers; }
	
	virtual const HttpHeadersCollection& headers() const { return m_headers; }
	
	virtual BodyStatus getBodyStatus() const { return m_body_status; }
	
	virtual void setBodyStatus(BodyStatus status) { m_body_status = status; }
	
	virtual std::size_t getBodySize() const { return m_body_size; }
	
	virtual void setBodySize(std::size_t size) { m_body_size = size; }
private:
	HttpStatusLine m_status_line;
	HttpHeadersCollection m_headers;
	BodyStatus m_body_status;
	std::size_t m_body_size;
};

#endif

