/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstatechunkbody.h"
#include "httpstream.h"
#include <algorithm>
#include <cstddef>
#include <cstring>

HttpStateChunkBody::HttpStateChunkBody(HttpStream* owner)
:	m_pOwner(owner), m_size(0), m_read(0)
{
}

void
HttpStateChunkBody::consumeDataChunk(const uint8_t*& begin, const uint8_t* end)
{
	if (m_read < m_size) {
		std::size_t block_size = std::min(std::size_t(end-begin), m_size-m_read);
		m_pOwner->eventGotContentPart(begin, begin+block_size);
		m_read += block_size;
		begin += block_size;
	}
	
	const uint8_t* nlpos = reinterpret_cast<const uint8_t*>(std::memchr(begin, '\n', end-begin));
	if (nlpos) {
		begin = nlpos+1;
		m_pOwner->switchState(HttpStream::STATE_CHUNK_HEADER);
	} else {
		begin = end;
	}
}

void
HttpStateChunkBody::consumeEOF()
{
	m_pOwner->eventError(HttpStream::ERR_UNEXPECTED_EOF);
	m_pOwner->switchState(HttpStream::STATE_ERROR);
}

void
HttpStateChunkBody::cleanup()
{
	m_size = 0;
	m_read = 0;
}

