/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstatechunkheader.h"
#include "httpstream.h"
#include <cstddef>
#include <cstring>

HttpStateChunkHeader::HttpStateChunkHeader(HttpStream* owner)
:	m_pOwner(owner),
	m_state(ST_LEFT_PADDING),
	m_chunkSize(0),
	m_chunkSizeDigits(0)
{
}

void
HttpStateChunkHeader::consumeDataChunk(const uint8_t*& begin, const uint8_t* end)
{
	uint8_t ch;
	switch (m_state) {
		case ST_LEFT_PADDING: {
			for (; begin < end; ++begin) {
				ch = *begin;
				if (ch != ' ' && ch != '\t') {
					m_state = ST_CHUNK_SIZE;
					goto case_ST_CHUNK_SIZE;
				}
			}
			break;
		}
		case ST_CHUNK_SIZE:
		case_ST_CHUNK_SIZE: {
			ch = *begin;
			if (ch >= uint8_t('0') && ch <= uint8_t('9')) {
				m_chunkSize = (m_chunkSize << 4) + (ch - uint8_t('0'));
			} else if (ch >= 'a' && ch <= 'f') {
				m_chunkSize = (m_chunkSize << 4) + 10 + (ch - uint8_t('a'));
			} else if (ch >= 'A' && ch <= 'F') {
				m_chunkSize = (m_chunkSize << 4) + 10 + (ch - uint8_t('A'));
			} else {
				m_state = ST_RIGHT_PADDING;
				goto case_ST_RIGHT_PADDING;
			}
			++begin;
			++m_chunkSizeDigits;
			break;
		}
		case ST_RIGHT_PADDING:
		case_ST_RIGHT_PADDING: {
			for (; begin < end; ++begin) {
				ch = *begin;
				if (ch == '\r' || ch == ' ' || ch == '\t') {
					continue;
				} else if (ch == '\n') {
					++begin;
					processChunkSize();
					break;
				} else if (ch == ';') {
					++begin;
					m_state = ST_ARGUMENTS;
					goto case_ST_ARGUMENTS;
				} else {
					m_pOwner->eventError(HttpStream::ERR_PARSING_CHUNK_HEADER);
					m_pOwner->switchState(HttpStream::STATE_ERROR);
				}
			}
			break;
		}
		case ST_ARGUMENTS:
		case_ST_ARGUMENTS: {
			const uint8_t* nl = reinterpret_cast<const uint8_t*>(std::memchr(begin, '\n', end-begin));
			if (nl) {
				begin = nl + 1;
				processChunkSize();
			} else {
				begin = end;
			}
			break;
		}
	}
}

void
HttpStateChunkHeader::consumeEOF()
{
	m_pOwner->eventError(HttpStream::ERR_UNEXPECTED_EOF);
	m_pOwner->switchState(HttpStream::STATE_ERROR);
}

void
HttpStateChunkHeader::cleanup()
{
	m_state = ST_LEFT_PADDING;
	m_chunkSize = 0;
	m_chunkSizeDigits = 0;
}

void
HttpStateChunkHeader::processChunkSize()
{
	if (m_chunkSizeDigits == 0) {
		m_pOwner->eventError(HttpStream::ERR_PARSING_CHUNK_HEADER);
		m_pOwner->switchState(HttpStream::STATE_ERROR);
	} else if (m_chunkSize == 0) {
		m_pOwner->switchState(HttpStream::STATE_FOOTERS);
	} else {
		m_pOwner->m_stateChunkBody.setChunkSize(m_chunkSize);
		m_pOwner->switchState(HttpStream::STATE_CHUNK_BODY);
	}
}

