/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstateflatbody.h"
#include "httpstream.h"
#include <cstddef>

HttpStateFlatBody::HttpStateFlatBody(HttpStream* owner)
:	m_pOwner(owner), m_isBodySizeDefined(false), m_bodySize(0), m_read(0)
{
}

void
HttpStateFlatBody::consumeDataChunk(const uint8_t*& begin, const uint8_t* end)
{
	if (m_isBodySizeDefined && m_read+std::size_t(end-begin) >= m_bodySize) {
		const uint8_t* to = begin+m_bodySize-m_read;
		m_pOwner->eventGotContentPart(begin, to);
		begin = to;
		m_pOwner->switchState(HttpStream::STATE_INITIAL);
		m_pOwner->eventMessageEnds(m_pOwner->m_isConnectionToBeClosed);
	} else {
		m_pOwner->eventGotContentPart(begin, end);
		m_read += end-begin;
		begin = end;
	}
}

void
HttpStateFlatBody::consumeEOF()
{
	if (m_isBodySizeDefined) {
		m_pOwner->eventError(HttpStream::ERR_UNEXPECTED_EOF);
		m_pOwner->switchState(HttpStream::STATE_ERROR);
	} else {
		m_pOwner->switchState(HttpStream::STATE_INITIAL);
		m_pOwner->eventMessageEnds(true);
	}
}

void
HttpStateFlatBody::cleanup()
{
	m_isBodySizeDefined = false;
	m_bodySize = 0;
	m_read = 0;
}

void
HttpStateFlatBody::setBodySize(std::size_t size)
{
	m_isBodySizeDefined = true;
	m_bodySize = size;
}

