/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPSTATEFOOTERS_H_
#define HTTPSTATEFOOTERS_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "httpstate.h"
#include "httpheaderscollection.h"
#include <vector>
#include <cstddef>

class HttpStream;

class HttpStateFooters : public HttpState
{
public:
	HttpStateFooters(HttpStream* owner);
	virtual void consumeDataChunk(const uint8_t*& begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual void cleanup();
private:
	enum State {
		ST_CHECKING_FOR_END,
		ST_LOOKING_FOR_COLON,
		ST_LOOKING_FOR_NEWLINE,
		ST_CHECKING_FOR_LWS,
		ST_CONSUMING_LWS
	};
private:
	void bufAppend(const uint8_t* begin, const uint8_t* end);
	void addHeader(const uint8_t* begin, const uint8_t* end);
	HttpStream* m_pOwner;
	State m_state;
	HttpHeadersCollection m_headers;
	std::vector<uint8_t> m_buf;
	std::size_t m_colonPos;
};

inline void
HttpStateFooters::bufAppend(const uint8_t* begin, const uint8_t* end)
{
	std::size_t bufpos = m_buf.size();
	m_buf.resize(end-begin+bufpos);
	std::memcpy(&m_buf[bufpos], begin, end-begin);
}

#endif

