/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstatusline.h"
#include "stringutils.h"
#include <sstream>

std::map<int, const char *> HttpStatusLine::m_message_map = HttpStatusLine::createMessageMap();

HttpStatusLine::HttpStatusLine(const HttpVersion& http_version, int code)
:	m_httpVersion(http_version), m_code(code)
{
	std::map<int, const char *>::const_iterator fit = m_message_map.find(code);
	if (fit != m_message_map.end()) {
		m_message = fit->second;
	}
}

HttpStatusLine::HttpStatusLine(const HttpVersion& http_version, int code, const std::string& message)
:	m_httpVersion(http_version), m_code(code), m_message(message)
{}

void
HttpStatusLine::setCodeAndMessage(int code)
{
	m_code = code;
	std::map<int, const char *>::const_iterator fit = m_message_map.find(code);
	if (fit != m_message_map.end()) {
		m_message = fit->second;
	} else {
		m_message.erase();
	}
}

std::string
HttpStatusLine::toString() const
{
	std::ostringstream strm;
	toStream(strm);
	return strm.str();
}

void
HttpStatusLine::toStream(std::ostream& strm) const
{
	if (m_httpVersion >= HttpVersion::HTTP_1_0) {
		strm << m_httpVersion << ' ';
	}
	strm << m_code << ' ' << m_message;
}

std::map<int, const char *>
HttpStatusLine::createMessageMap()
{
	std::map<int, const char *> res;
	res[SC_CONTINUE] = "Continue";
	res[SC_SWITCHING_PROTOCOLS] = "Switching Protocols";
	res[SC_OK] = "OK";
	res[SC_CREATED] = "Created";
	res[SC_ACCEPTED] = "Accepted";
	res[SC_NON_AUTHORITATIVE_INFORMATION] = "Non-Authoritative Information";
	res[SC_NO_CONTENT] = "No Content";
	res[SC_RESET_CONTENT] = "Reset Content";
	res[SC_PARTIAL_CONTENT] = "Partial Content";
	res[SC_MULTIPLE_CHOICES] = "Multiple Choices";
	res[SC_MOVED_PERMANENTLY] = "Moved Permanently";
	res[SC_FOUND] = "Found";
	res[SC_SEE_OTHER] = "See Other";
	res[SC_NOT_MODIFIED] = "Not Modified";
	res[SC_USE_PROXY] = "Use Proxy";
	res[SC_TEMPORARY_REDIRECT] = "Temporary Redirect";
	res[SC_BAD_REQUEST] = "Bad Request";
	res[SC_UNAUTHORIZED] = "Unauthorized";
	res[SC_PAYMENT_REQUIRED] = "Payment Required";
	res[SC_FORBIDDEN] = "Forbidden";
	res[SC_NOT_FOUND] = "Not Found";
	res[SC_METHOD_NOT_ALLOWED] = "Method Not Allowed";
	res[SC_NOT_ACCEPTABLE] = "Not Acceptable";
	res[SC_PROXY_AUTHENTICATION_REQUIRED] = "Proxy Authentication Required";
	res[SC_REQUEST_TIMEOUT] = "Request Timeout";
	res[SC_CONFLICT] = "Conflict";
	res[SC_GONE] = "Gone";
	res[SC_LENGTH_REQUIRED] = "Length Required";
	res[SC_PRECONDITION_FAILED] = "Precondition Failed";
	res[SC_REQUEST_ENTITY_TOO_LARGE] = "Request Entity Too Large";
	res[SC_REQUEST_URI_TOO_LONG] = "Request-URI Too Long";
	res[SC_UNSUPPORTED_MEDIA_TYPE] = "Unsupported Media Type";
	res[SC_REQUESTED_RANGE_NOT_SATISFIABLE] = "Requested Range Not Satisfiable";
	res[SC_EXPECTATION_FAILED] = "Expectation Failed";
	res[SC_INTERNAL_SERVER_ERROR] = "Internal Server Error";
	res[SC_NOT_IMPLEMENTED] = "Not Implemented";
	res[SC_BAD_GATEWAY] = "Bad Gateway";
	res[SC_SERVICE_UNAVAILABLE] = "Service Unavailable";
	res[SC_GATEWAY_TIMEOUT] = "Gateway Timeout";
	res[SC_HTTP_VERSION_NOT_SUPPORTED] = "HTTP Version Not Supported";
	return res;
}

