/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstream.h"

HttpStream::HttpStream()
:	m_stateInitial(this), m_stateStartLine(this), m_stateHeaders(this),
	m_stateFlatBody(this), m_stateChunkHeader(this), m_stateChunkBody(this),
	m_stateFooters(this), m_stateSimpleResponse(this), m_stateError(this),
	m_pCurState(&m_stateInitial), m_isInsideConsumeFunction(false),
	m_isConnectionToBeClosed(false)
{
	m_stateTable[STATE_INITIAL] = &m_stateInitial;
	m_stateTable[STATE_START_LINE] = &m_stateStartLine;
	m_stateTable[STATE_HEADERS] = &m_stateHeaders;
	m_stateTable[STATE_FLAT_BODY] = &m_stateFlatBody;
	m_stateTable[STATE_CHUNK_HEADER] = &m_stateChunkHeader;
	m_stateTable[STATE_CHUNK_BODY] = &m_stateChunkBody;
	m_stateTable[STATE_FOOTERS] = &m_stateFooters;
	m_stateTable[STATE_SIMPLE_RESPONSE] = &m_stateSimpleResponse;
	m_stateTable[STATE_ERROR] = &m_stateError;
}

void
HttpStream::consumeDataChunk(const uint8_t* begin, const uint8_t* end)
{
	try {
		while (begin != end) {
			m_isInsideConsumeFunction = true;
			m_pCurState->consumeDataChunk(begin, end);
			m_isInsideConsumeFunction = false;
		}
	} catch (State state) {
		m_isInsideConsumeFunction = false;
		switchState(state);
	} catch (...) {
		m_isInsideConsumeFunction = false;
		switchState(STATE_INITIAL);
		throw;
	}
}

void
HttpStream::consumeEOF()
{
	try {
		m_isInsideConsumeFunction = true;
		m_pCurState->consumeEOF();
		m_isInsideConsumeFunction = false;
	} catch (State state) {
		m_isInsideConsumeFunction = false;
		switchState(state);
	} catch (...) {
		m_isInsideConsumeFunction = false;
		switchState(STATE_INITIAL);
		throw;
	}
}

void
HttpStream::resetState()
{
	if (m_isInsideConsumeFunction) {
		throw STATE_INITIAL;
	} else {
		switchState(STATE_INITIAL);
	}
}

bool
HttpStream::isInitialState() const
{
	// gcc 2.95.x needs this cast
	return (m_pCurState == static_cast<const HttpState *>(&m_stateInitial));
}

bool
HttpStream::isErrorState() const
{
	// gcc 2.95.x needs this cast
	return (m_pCurState == static_cast<const HttpState *>(&m_stateError));
}

void
HttpStream::switchState(State state)
{
	m_pCurState->cleanup();
	m_pCurState = m_stateTable[state];
}

/*
void
HttpStream::jumpToState(State state)
{
	if (m_insideConsumeFunction) {
		throw state;
	} else {
		switchState(state);
	}
}
*/

