/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPSTREAM_H_
#define HTTPSTREAM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "httpstate.h"
#include "httpstatestartline.h"
#include "httpstateheaders.h"
#include "httpstateinitial.h"
#include "httpstateflatbody.h"
#include "httpstatechunkheader.h"
#include "httpstatechunkbody.h"
#include "httpstatefooters.h"
#include "httpstatesimpleresponse.h"
#include "httpstateerror.h"
#include "httpheaderscollection.h"
#include "httprequestmetadata.h"
#include "httpresponsemetadata.h"
#include "httpversion.h"
#include <cstddef>

class HttpStream
{
	friend class HttpStateInitial;
	friend class HttpStateStartLine;
	friend class HttpStateHeaders;
	friend class HttpStateFlatBody;
	friend class HttpStateChunkHeader;
	friend class HttpStateChunkBody;
	friend class HttpStateFooters;
	friend class HttpStateSimpleResponse;
	friend class HttpStateError;
public:
	HttpStream();
	virtual ~HttpStream() {}
	void consumeDataChunk(const uint8_t *begin, const uint8_t* end);
	void consumeEOF();
	void resetState();
	bool isInitialState() const;
	bool isErrorState() const;
protected:
	enum ErrorCode {
		ERR_PARSING_REQUEST_LINE = 0,
		ERR_PARSING_HEADER,
		ERR_PARSING_CHUNK_HEADER,
		ERR_LENGTH_REQUIRED,
		ERR_UNEXPECTED_EOF,
		NUMBER_OF_ERRORS
	};
	virtual bool isRequestStream() const = 0;
	virtual bool isResponseOnHeadRequest() const = 0;
	virtual void eventError(int ecode, const char *info=0) = 0;
	virtual void eventGotMetadata(HttpRequestMetadata& metadata) = 0;
	virtual void eventGotMetadata(HttpResponseMetadata& metadata) = 0;
	virtual void eventGotContentPart(const uint8_t* begin, const uint8_t* end) = 0;
	virtual void eventGotFooters(const HttpHeadersCollection& footers) = 0;
	virtual void eventMessageEnds(bool connection_needs_closing) = 0;
private:
	enum State {
		STATE_INITIAL = 0,
		STATE_START_LINE,
		STATE_HEADERS,
		STATE_FLAT_BODY,
		STATE_CHUNK_HEADER,
		STATE_CHUNK_BODY,
		STATE_FOOTERS,
		STATE_SIMPLE_RESPONSE,
		STATE_ERROR,
		NUMBER_OF_STATES
	};
private:
	void switchState(State state);
	HttpStateInitial m_stateInitial;
	HttpStateStartLine m_stateStartLine;
	HttpStateHeaders m_stateHeaders;
	HttpStateFlatBody m_stateFlatBody;
	HttpStateChunkHeader m_stateChunkHeader;
	HttpStateChunkBody m_stateChunkBody;
	HttpStateFooters m_stateFooters;
	HttpStateSimpleResponse m_stateSimpleResponse;
	HttpStateError m_stateError;
	HttpState* m_pCurState;
	HttpState* m_stateTable[NUMBER_OF_STATES];
	bool m_isInsideConsumeFunction;
	bool m_isConnectionToBeClosed;
};

#endif

