/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPSTREAMWRITER_H_
#define HTTPSTREAMWRITER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "httpmessagemetadata.h"
#include <cstddef>

class HttpStreamWriter
{
public:
	class Receiver
	{
		friend class HttpStreamWriter;
	protected:
		virtual void moreData(const uint8_t* begin, const uint8_t* end) = 0;
		virtual void done(bool conn_close) = 0;
	};
	HttpStreamWriter(Receiver* receiver, HttpMessageMetadata& metadata,
		bool chunked_ok, bool force_conn_close);
private:
	HttpStreamWriter(const HttpStreamWriter&);            //
	HttpStreamWriter& operator=(const HttpStreamWriter&); // not implemented
public:
	void write(const std::string& data);
	void write(const uint8_t* begin, const uint8_t* end);
	void end();
private:
	enum {
		BUF_SIZE = 1440,
		CHUNK_SIZE_DIGITS = 3,
		DATA_BEGIN = CHUNK_SIZE_DIGITS + 2, // 2 for \r\n
		DATA_END = BUF_SIZE - 2 // 2 for \r\n
	};
	
	void outputString(const std::string& str);
	void writeChunkHeader(std::size_t size);
	
	Receiver* m_pReceiver;
	bool m_noBody;
	bool m_isChunkedOutput;
	bool m_isConnectionToBeClosed;
	uint8_t m_buffer[BUF_SIZE];
	std::size_t m_offset;
};

#endif

