/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpversion.h"
#include "stringutils.h"
#include <cstring>
#include <sstream>

const HttpVersion HttpVersion::HTTP_0_9(0, 9);
const HttpVersion HttpVersion::HTTP_1_0(1, 0);
const HttpVersion HttpVersion::HTTP_1_1(1, 1);

std::string
HttpVersion::toString() const
{
	std::ostringstream strm;
	toStream(strm);
	return strm.str();
}

void
HttpVersion::toStream(std::ostream& strm) const
{
	strm << "HTTP/" << m_major << '.' << m_minor;
}

bool
HttpVersion::parse(const std::string& str)
{
	const uint8_t* begin = reinterpret_cast<const uint8_t*>(str.data());
	return parse(begin, begin+str.length());
}

bool
HttpVersion::parse(const uint8_t* begin, const uint8_t* end)
{
	if (end-begin < 8 || std::memcmp(begin, "HTTP/", 5)) {
		return false;
	}
	begin += 5;
	const uint8_t* dot_pos = reinterpret_cast<const uint8_t*>(std::memchr(begin, '.', end-begin));
	if (!dot_pos || dot_pos == begin || dot_pos == end-1) {
		return false;
	}
	const uint8_t* num_end = dot_pos;
	int major = StringUtils::parseUnsigned(begin, num_end);
	if (num_end != dot_pos) {
		return false;
	}
	num_end = end;
	int minor = StringUtils::parseUnsigned(dot_pos+1, num_end);
	if (num_end != end) {
		return false;
	}
	m_major = major;
	m_minor = minor;
	return true;
}

bool
HttpVersion::operator<(const HttpVersion& other) const
{
	if (m_major < other.m_major) {
		return true;
	} else if (m_major > other.m_major) {
		return false;
	} else {
		return (m_minor < other.m_minor);
	}
}

bool
HttpVersion::operator>(const HttpVersion& other) const
{
	if (m_major > other.m_major) {
		return true;
	} else if (m_major < other.m_major) {
		return false;
	} else {
		return (m_minor > other.m_minor);
	}
}

