/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGEINSPECTOR_H_
#define IMAGEINSPECTOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractimageinspector.h"
#include <vector>
#include <utility>
#include <memory>

class ImageInspector : public AbstractImageInspector
{
public:
	enum ImageType { T_JPEG, T_GIF, T_PNG, T_UNKNOWN };
	ImageInspector();
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual void reset();
	virtual std::pair<const uint8_t*, const uint8_t*> getAccumulatedData() const;
	virtual bool isInspectionInProgress() const;
	virtual bool isInspectionFailed() const;
	virtual int getWidth() const;
	virtual int getHeight() const;
	virtual bool isAnimated() const;
	ImageType getImageType() const { return m_type; }
private:
	enum State { ST_SIGNATURE, ST_DELEGATION, ST_UNKNOWN_TYPE };
	bool considerType(ImageType type, const uint8_t* begin,
		const uint8_t* end, const uint8_t* sig, std::size_t sig_size);
	State m_state;
	ImageType m_type;
	uint8_t m_typesBeingConsidered;
	std::vector<uint8_t> m_data;
	std::auto_ptr<AbstractImageInspector> m_ptrDelegate;
};

#endif

