/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGERESPONSEFILTER_H_
#define IMAGERESPONSEFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "abstractresponsefilter.h"
#include "httpresponsemetadata.h"
#include "imageinspector.h"
#include "conf.h"
#include <sigc++/sigc++.h>

class ImageResponseFilter : public AbstractResponseFilter, public SigC::Object
{
public:
	struct Params
	{
		Params(const Config* conf) : config(conf) {}
		const Config* config;
		int ad_rule;
		int width, height;
		std::string target;
		bool external_link;
		bool external_img;
		bool suspicious_link;
		bool suspicious_img;
		bool redir;
		bool cookies;
	};
	
	static AbstractResponseFilter* create(NextFilterProvider* follower,
		HttpResponseMetadata* metadata, const Params& params);
private:
	ImageResponseFilter(NextFilterProvider* follower,
		HttpResponseMetadata* metadata, const Params* params);
	
	ImageResponseFilter(const ImageResponseFilter&);            //
	
	ImageResponseFilter& operator=(const ImageResponseFilter&); // not implemented
public:
	virtual ~ImageResponseFilter();
	
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	
	virtual void consumeEOF();
private:
	bool isAd() const;
	
	void performSubstitution();
	
	static AbstractResponseFilter* construct(HttpResponseMetadata* metadata,
		NextFilterProvider* follower, const Params* params);
private:
	enum State { ST_INSPECTING, ST_FORWARDING, ST_SUBSTITUTING };
	NextFilterProvider m_followerProvider;
	HttpResponseMetadata m_metadata;
	Params m_params;
	State m_state;
	AbstractResponseFilter* m_pFollower;
	ImageInspector m_inspector;
};

#endif

