/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IPCMESSAGE_H_
#define IPCMESSAGE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include <string>
#include <sstream>
#include <vector>

class IPCMessage
{
public:
	IPCMessage() {}
	IPCMessage(const std::string& method) : m_method(method) {}
	const std::string& getMethod() const { return m_method; }
	void setMethod(const std::string& method) { m_method = method; }
	void appendParameter(const std::string& param) { m_params.push_back(param); }
	void appendParameter(const uint8_t* begin, const uint8_t* end)
	{ m_params.push_back(std::string((const char*)begin, (const char*)end)); }
	void appendParameter(int param)
	{ std::ostringstream strm; strm << param; appendParameter(strm.str()); }
	void clearParams() { m_params.clear(); }
	int nParams() const { return m_params.size(); }
	std::string getParameter(int pos) const { return (pos < nParams() ? m_params[pos] : ""); }
	int getParameterAsInt(int pos) const
	{ int res = 0; std::istringstream strm(getParameter(pos)); strm >> res; return res; }
	std::string serialize() const;
	bool unserialize(const std::string& str);
private:
	std::string readBytes(std::istream& strm, std::string::size_type bytes);
	std::string m_method;
	std::vector<std::string> m_params;
};

#endif
