/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IPCPORTAL_H_
#define IPCPORTAL_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include <memory>
#include <unistd.h>
#include "ipcmessage.h"

class IPCPortal
{
public:
	IPCPortal();
	IPCPortal(int rfd, int wfd);
private:
	IPCPortal(const IPCPortal&);            //
	IPCPortal& operator=(const IPCPortal&); // not implemented
public:
	const IPCMessage *firstPendingMessage() const;
	const IPCMessage *firstPendingRequest() const;
	void dropFirstPendingMessage() { m_receivedMessages.pop_front(); }
	void processFirstPendingRequest(const IPCMessage& response);
	void sendMessage(const IPCMessage& msg);
	bool sendRequest(const IPCMessage& msg, IPCMessage& response);
	bool hasDataToSend() const { return !m_dataToSend.empty(); }
	bool flushPendingData();
	void readChunk(const std::string& data);
	bool readChunk();
	bool writeChunk();
	bool readWontBlock();
	bool writeWontBlock();
	int getReadFD() const { return m_rfd; }
	int getWriteFD() const { return m_wfd; }
	bool isOperational() const { return (m_rfd != -1 && m_wfd != -1); }
private:
	class Message
	{
	public:
		enum Type { T_ASYNC, T_REQUEST, T_RESPONSE, T_END };
		Message() : m_type(T_ASYNC), m_id(0) {}
		Message(Type type, unsigned int id, const IPCMessage& msg)
		: m_type(type), m_id(id), m_msg(msg) {}
		Type getType() const { return m_type; }
		unsigned int getId() const { return m_id; }
		IPCMessage& message() { return m_msg; }
		const IPCMessage& message() const { return m_msg; }
		std::string serialize() const;
		bool unserialize(const std::string& data);
	private:
		Type m_type;
		unsigned int m_id;
		IPCMessage m_msg;
	};
	int m_rfd, m_wfd;
	unsigned int m_lastMessageId;
	std::list<Message> m_receivedMessages;
	std::list<Message> m_receivedRequests;
	std::auto_ptr<Message> m_pResponse;
	std::string m_dataToSend;
	bool m_receivingHeader;
	std::string m_bufferedData;
	std::string::size_type m_bodySize;
};

#endif

