/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef JSENVIRONMENT_H_
#define JSENVIRONMENT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef WIN32
#define XP_WIN
#else
#define XP_UNIX
#endif

#include "types.h"
#include <string>
#include <map>
#include <sigc++/sigc++.h>
#include <jsapi.h>


class JSEnvironment
{
public:
	typedef SigC::Signal2<void, const uint8_t*, const uint8_t*> DataOutputSignal;
	typedef SigC::Signal0<void> PopupSignal;
	JSEnvironment();
	~JSEnvironment();
	bool executeScript(const std::string& script, const std::string& js_version="");
	bool executeScriptAsFunction(const std::string& script, const std::string& js_version="");
	void addForm(const std::string& name, const std::string& action, const std::string& target);
	DataOutputSignal& onDataOutput() { return m_dataOutputSignal; }
	PopupSignal& onPopup() { return m_popupSignal; }
private:
	JSEnvironment(const JSEnvironment&);            //
	JSEnvironment& operator=(const JSEnvironment&); // not implemented
	
	void setJSVersion(const std::string& version="");
	void executeScheduledActions();
	static JSEnvironment *getEnvironment(JSContext *cx);
	static void errorReporter(JSContext *cx, const char *msg, JSErrorReport *report);
	static JSBool branchCallback(JSContext *cx, JSScript *script);
	static JSBool cookieGetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
	static JSBool cookieSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
	static JSBool onloadSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
	static JSBool onunloadSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
	static JSBool javaEnabled(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool windowOpen(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool setTimeout(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool documentWrite(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool documentWriteln(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool imageConstructor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool formSubmit(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool doNothing(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSBool bfilterDebugLog(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
	static JSClass m_generic_class;
	DataOutputSignal m_dataOutputSignal;
	PopupSignal m_popupSignal;
	JSRuntime *m_runtime;
	JSContext *m_ctx;
	JSClass m_global_class;
	JSClass m_document_class;
	JSClass m_navigator_class;
	JSClass m_location_class;
	JSClass m_screen_class;
	JSClass m_image_class;
	JSClass m_form_class;
	JSClass m_mimetype_class;
	JSClass m_plugin_class;
	JSClass m_body_class;
	JSObject *m_global_obj;
	JSObject *m_doc_obj;
	JSObject *m_form_array;
	int m_branch_cnt;
	std::string m_document_cookie;
	std::multimap<int, std::string> m_scheduled_actions;
	static const int BRANCH_LIMIT = 1000;
};

#endif

