/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <string>
#include <popt.h>
#include "network.h"
#include <ipcportal.h>

// some older versions of popt don't define POPT_TABLEEND
#ifndef POPT_TABLEEND
#define POPT_TABLEEND { NULL, '\0', 0, 0, 0, NULL, NULL }
#endif

int main_unix_standalone(const std::string& confdir, bool nodaemon);
int main_unix_backend(Network::Socket csock, IPCPortal* portal);

int main(int argc, char *argv[])
{
	enum {
		ARG_VERSION = 1,
		ARG_CONFDIR,
		ARG_NODAEMON,
		ARG_BACKEND
	};
	bool backend = false;
	bool nodaemon = false;
	char *cdir = 0;
	std::string confdir = CONFDIR;
	struct poptOption options[] = {
		{ "version", 'v', POPT_ARG_NONE, NULL, ARG_VERSION, "Print version and exit" },
		{ "confdir", 'c', POPT_ARG_STRING, &cdir, ARG_CONFDIR, "Set custom config directory", "dir" },
		{ "nodaemon", 'n', POPT_ARG_NONE, NULL, ARG_NODAEMON, "Disable background daemon mode" },
		{ "backend", '\0', POPT_ARG_NONE|POPT_ARGFLAG_DOC_HIDDEN, NULL, ARG_BACKEND },
		POPT_AUTOHELP
		POPT_TABLEEND
	};
	poptContext context = poptGetContext(NULL, argc, (const char**)argv, options, 0);
	int arg = 0;
	while ((arg = poptGetNextOpt(context)) > 0) {
		switch (arg) {
			case ARG_VERSION:
				std::cout << VERSION << std::endl;
				return 0;
			case ARG_CONFDIR:
				confdir = cdir;
				break;
			case ARG_NODAEMON:
				nodaemon = true;
				break;
			case ARG_BACKEND:
				backend = true;
				break;
		}
	}
	if (arg != -1) {
		std::cerr << poptBadOption(context, POPT_BADOPTION_NOALIAS)
			<< ": " << poptStrerror(arg) << std::endl;
		return 1;
	}
	poptFreeContext(context);
	
	if (backend) {
		IPCPortal portal(0, 1);
		return main_unix_backend(3, &portal);
	} else {
		return main_unix_standalone(confdir, nodaemon);
	}
}

