/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MASTERPTR_H_
#define MASTERPTR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

template<typename T>
class MasterPtr
{
public:
	MasterPtr() : m_pObj(0) {}
	
	explicit
	MasterPtr(T* obj) : m_pObj(obj) {}
	
	MasterPtr(const MasterPtr& other);
	
	template<typename OT>
	MasterPtr(const MasterPtr<OT>& other);
	
	~MasterPtr();
public:
	MasterPtr& operator=(const MasterPtr& rhs);
	
	template<typename OT>
	MasterPtr& operator=(const MasterPtr<OT>& rhs);
	
	T& operator*() const { return *m_pObj; }
	
	T* operator->() const { return m_pObj; }
	
	bool operator==(const MasterPtr& rhs) {
		return get() == rhs.get();
	}
	
	bool operator!=(const MasterPtr& rhs) {
		return get() != rhs.get();
	}
	
	private:
	typedef void (MasterPtr::*MemberPtr)(T*);
public:
	operator MemberPtr() const {
		return m_pObj ? &MasterPtr::reset : 0;
	}
	
	T* get() const { return m_pObj; }
	
	T* release();
	
	void reset(T* obj);
	
	void swap(MasterPtr& other);
private:
	T* m_pObj;
};


template<typename T>
MasterPtr<T>::MasterPtr(const MasterPtr& other)
{
	if (other.m_pObj) {
		m_pObj = other.m_pObj->clone();
	} else {
		m_pObj = 0;
	}
}

template<typename T>
template<typename OT>
MasterPtr<T>::MasterPtr(const MasterPtr<OT>& other)
{
	if (other.m_pObj) {
		m_pObj = other.m_pObj->clone();
	} else {
		m_pObj = 0;
	}
}

template<typename T>
MasterPtr<T>::~MasterPtr()
{
	if (m_pObj)
		delete m_pObj;
}

template<typename T>
MasterPtr<T>&
MasterPtr<T>::operator=(const MasterPtr& rhs)
{
	MasterPtr(rhs).swap(*this);
	return *this;
}

template<typename T>
template<typename OT>
MasterPtr<T>&
MasterPtr<T>::operator=(const MasterPtr<OT>& rhs)
{
	MasterPtr(rhs).swap(*this);
	return *this;
}

template<typename T>
T*
MasterPtr<T>::release()
{
	T* obj;
	if ((obj = m_pObj)) {
		delete m_pObj;
		m_pObj = 0;
	}
	return obj;
}

template<typename T>
void
MasterPtr<T>::reset(T* obj)
{
	MasterPtr(obj).swap(*this);
}

template<typename T>
void
MasterPtr<T>::swap(MasterPtr& other)
{
	T* obj = other.m_pObj;
	other.m_pObj = m_pObj;
	m_pObj = obj;
}

#endif

