/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef NETWORK_H_
#define NETWORK_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "networkerror.h"
#include <cstdio>
#include <cstring>
#include <string>
#include <list>
#include <map>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/types.h>
#include <sys/uio.h>
#include <netdb.h>
#include <netinet/in.h>
#endif

class Network
{
public:
#ifdef WIN32
	typedef SOCKET Socket;
	static const Socket INVALID_SOCK = INVALID_SOCKET;
	struct IoVec
	{
		// binary-compatible with WSABUF
		u_long iov_len;
		char* iov_base;
	};
#else
	typedef int Socket;
	static const Socket INVALID_SOCK = -1;
	typedef iovec IoVec;
#endif
	typedef std::list<in_addr> addr_list_type;
	static addr_list_type resolve(const char *host, NetworkError *err=0);
	static Socket tcpServer(const char *host, unsigned port, NetworkError* err=0);
	static Socket tcpServerWaitConn(Socket sock, struct sockaddr_in *client, NetworkError* err=0);
	static Socket tcpConnect(const char *host, unsigned port, NetworkError* err=0);
	static bool tcpSendData(Socket sock, const std::string& data, NetworkError* err=0);
	static void sockSetNodelay(Socket sock, bool state);
	static int writev(int sock, const IoVec* iov, int cnt);
	static void closeSocket(Socket sock);
	static void disconnectAndCloseSocket(Socket sock, int timeout);
	static bool isValidSocket(Socket sock);
	static std::string getLastError();
private:
#ifndef WIN32
	static std::map<std::string, addr_list_type> m_resolver_cache;
#endif
};

#endif

