/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "pngimageinspector.h"
#include <cstring>

const uint8_t PNGImageInspector::SIGNATURE[8] = { 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a };

PNGImageInspector::PNGImageInspector()
:	m_state(ST_SIGNATURE), m_isInProgress(true),
	m_isFailed(false), m_width(0), m_height(0)
{
}

void
PNGImageInspector::consumeDataChunk(const uint8_t* begin, const uint8_t* end)
{
	std::size_t size = m_data.size();
	m_data.resize(end-begin+size);
	std::memcpy(&m_data[size], begin, end-begin);
	size = m_data.size();
	if (!m_isInProgress) {
		return;
	}
	switch (m_state) {
		case ST_SIGNATURE: {
			if (size < sizeof(SIGNATURE)) {
				return;
			}
			if (std::memcmp(&m_data[0], SIGNATURE, sizeof(SIGNATURE))) {
				m_isInProgress = false;
				m_isFailed = true;
				return;
			} else {
				m_state = ST_HEADER;
			}
		}
		case ST_HEADER: {
			if (size < 24) {
				return;
			}
			const uint8_t *bytes = &m_data[0];
			m_width = ((unsigned)bytes[16]<<24)+((unsigned)bytes[17]<<16)+
				((unsigned)bytes[18]<<8)+(unsigned)bytes[19];
			m_height = ((unsigned)bytes[20]<<24)+((unsigned)bytes[21]<<16)+
				((unsigned)bytes[22]<<8)+(unsigned)bytes[23];
			m_isInProgress = false;
		}
	}
}

void
PNGImageInspector::consumeEOF()
{
	if (m_isInProgress) {
		m_isInProgress = false;
		m_isFailed = true;
	}
}

void
PNGImageInspector::reset()
{
	m_state = ST_SIGNATURE;
	std::vector<uint8_t>().swap(m_data);
	m_isInProgress = true;
	m_isFailed = false;
	m_width = 0;
	m_height = 0;
}

std::pair<const uint8_t*, const uint8_t*>
PNGImageInspector::getAccumulatedData() const
{
	return std::pair<const uint8_t*, const uint8_t*>(&m_data[0], &m_data[m_data.size()]);
}

