/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef PNGIMAGEINSPECTOR_H_
#define PNGIMAGEINSPECTOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractimageinspector.h"
#include <vector>
#include <utility>

class PNGImageInspector : public AbstractImageInspector
{
public:
	PNGImageInspector();
	virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
	virtual void consumeEOF();
	virtual void reset();
	virtual std::pair<const uint8_t*, const uint8_t*> getAccumulatedData() const;
	virtual bool isInspectionInProgress() const { return m_isInProgress; }
	virtual bool isInspectionFailed() const { return m_isFailed; }
	virtual int getWidth() const { return m_width; }
	virtual int getHeight() const { return m_height; }
	virtual bool isAnimated() const { return false; }
	static const uint8_t SIGNATURE[8];
private:
	enum State { ST_SIGNATURE, ST_HEADER };
	State m_state;
	std::vector<uint8_t> m_data;
	bool m_isInProgress;
	bool m_isFailed;
	int m_width;
	int m_height;
};

#endif

