/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "replacementflash.h"
#include <cstring>

ReplacementFlash::ReplacementFlash(int width, int height, const char* orig_url,
                                   bool draw_border, const uint8_t* border_color)
:	m_bitoffset(0)
{
	if (width <= 0) {
		width = 1;
	} else if (width > 1600) {
		width = 1600;
	}
	if (height <= 0) {
		height = 1;
	} else if (height > 1600) {
		height = 1600;
	}
	m_width = width*20;   //
	m_height = height*20; // dimentions in twips
	
	m_data.reserve(512);
	m_data.resize(8);
	std::memcpy(&m_data[0], "FWS", 3);
	m_data[3] = 4; // flash version
	// length of the file (uint32_t) will be written later
	
	appendRect(0, m_width, 0, m_height); // frame dimentions
	append<uint16_t>(0x1000); // frame rate
	append<uint16_t>(1); // frame count
	
	append<uint16_t>((22<<6)|0x3f); // DefineShape2 tag
	std::size_t shape_length_pos = m_data.size();
	append<uint32_t>(0); // length will be written later
	append<uint16_t>(1); // char_id
	appendRect(0, m_width, 0, m_height);
	append<uint8_t>(1); // count of fill styles
	append<uint8_t>(0); // type of fill style: solid fill
	append<uint8_t>(0xff); // fill color (r)
	append<uint8_t>(0xff); // fill color (g)
	append<uint8_t>(0xff); // fill color (b)
	append<uint8_t>(1); // count of line styles
	append<uint16_t>(20); // line width in twips
	if (draw_border) {
		append<uint8_t>(border_color[0]);
		append<uint8_t>(border_color[1]);
		append<uint8_t>(border_color[2]);
	} else {
		append<uint8_t>(0xff);
		append<uint8_t>(0xff);
		append<uint8_t>(0xff);
	}
	append<uint8_t>(0x88); // number of fill style index bits (8), and line style index bits (8)
	appendBits(11, 6); // flags: non-edge record, move, change fill 0 style, change line style
	appendBits(20, 5); // move bit count: 20
	appendBits(10, 20); // move to: originX+10
	appendBits(10, 20); // move to: originY+10
	appendBits(1, 8); // fill 0 style: 1
	appendBits(1, 8); // line style: 1
	appendBits(0xfc, 8); // 11111100, horizontal line, num bits: 15+2
	appendBits(m_width-20, 17); // delta x
	appendBits(0xfd, 8); // 11111101, vertical line, num bits: 15+2
	appendBits(m_height-20, 17); // delta y
	appendBits(0xfc, 8);
	appendBits(-(m_width-20), 17);
	appendBits(0xfd, 8);
	appendBits(-(m_height-20), 17);
	appendBits(0, 6); // EndShapeRecord
	m_bitoffset = 0;
	write<uint32_t>(m_data.size()-shape_length_pos-4, shape_length_pos);
	
	append<uint16_t>((7<<6)|0x3f); // Button tag
	std::size_t button_length_pos = m_data.size();
	append<uint32_t>(0); // length will be written later
	append<uint16_t>(2); // char_id
	append<uint8_t>(0x0f); // flags: present in all states
	append<uint16_t>(1); // char_id to place
	append<uint16_t>(1); // place depth
	appendBits(0, 2); // hasScale: no, hasRotate: no
	appendBits(1, 5); // nTranslateBits: 1
	appendBits(0, 2); // xTranslate = 0, yTranslate = 0
	m_bitoffset = 0;
	append<uint8_t>(0); // CharacterEndFlag
	append<uint8_t>(0x83); // ActionGetURL
	std::size_t get_url_action_length_pos = m_data.size();
	append<uint16_t>(0); // length will be written later
	std::size_t url_len = strlen(orig_url);
	std::size_t url_pos = m_data.size();
	m_data.resize(url_pos+url_len+1);
	std::memcpy(&m_data[url_pos], orig_url, url_len+1);
	std::size_t target_pos = m_data.size();
	static const char target[] = "_level0";
	m_data.resize(target_pos+sizeof(target));
	std::memcpy(&m_data[target_pos], target, sizeof(target));
	write<uint16_t>(m_data.size()-get_url_action_length_pos-2, get_url_action_length_pos);
	append<uint8_t>(0); // ActionEndFlag
	write<uint32_t>(m_data.size()-button_length_pos-4, button_length_pos);
	
	append<uint16_t>((26<<6)|0x3f); // PlaceObject2 tag
	std::size_t place_object_length_pos = m_data.size();
	append<uint32_t>(0); // length will be written later
	append<uint8_t>(0x06); // place character, has matrix
	append<uint16_t>(1); // depth
	append<uint16_t>(2); // char_id to place
	appendBits(0, 2); // hasScale: no, hasRotate: no
	appendBits(1, 5); // nTranslateBits: 1
	appendBits(0, 2); // xTranslate = 0, yTranslate = 0
	m_bitoffset = 0;
	write<uint32_t>(m_data.size()-place_object_length_pos-4, place_object_length_pos);
	
	append<uint16_t>(1<<6); // show frame
	append<uint16_t>(0); // end tag
	
	write<uint32_t>(m_data.size(), 4);
}

void
ReplacementFlash::appendRect(int left, int right, int top, int bottom)
{
	static const int nbits = (1<<5)-1;
	appendBits(nbits, 5);
	appendBits(left, nbits);
	appendBits(right, nbits);
	appendBits(top, nbits);
	appendBits(bottom, nbits);
	m_bitoffset = 0;
}

