/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef REPLACEMENTFLASH_H_
#define REPLACEMENTFLASH_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include <utility>
#include <vector>

class ReplacementFlash
{
public:
	ReplacementFlash(int width, int height, const char* orig_url,
		bool draw_border, const uint8_t* border_color);
private:
	ReplacementFlash(const ReplacementFlash&);            //
	ReplacementFlash& operator=(const ReplacementFlash&); // not implemented
public:
	std::pair<const uint8_t*, const uint8_t*> getBinaryData() const {
		return std::pair<const uint8_t*, const uint8_t*>(&m_data[0], &m_data[m_data.size()]);
	}
	static const char* getContentType() { return "application/x-shockwave-flash"; }
private:
	template<typename T> void write(T value, std::size_t pos);
	template<typename T> void append(T value);
	template<typename T> void appendBits(T value, int nbits);
	void appendRect(int left, int right, int top, int bottom);
private:
	std::vector<uint8_t> m_data;
	int m_bitoffset;
	int m_width;
	int m_height;
};

template<typename T> void
ReplacementFlash::write(T value, std::size_t pos)
{
	uint8_t* ptr = &m_data[pos];
	for (unsigned int shift = 0; shift < sizeof(T)*8; shift+=8) {
		*ptr++ = (value>>shift);
	}
}

template<typename T> void
ReplacementFlash::append(T value)
{
	std::size_t pos = m_data.size();
	m_data.resize(pos+sizeof(T));
	write(value, pos);
}

template<typename T> void
ReplacementFlash::appendBits(T value, int nbits)
{
	std::size_t pos = m_data.size();
	if (m_bitoffset > 0) {
		int bits_avail = 8-m_bitoffset;
		if (nbits <= bits_avail) {
			bits_avail -= nbits;
			m_data[pos-1] |= ((value&((1<<nbits)-1))<<bits_avail);
			m_bitoffset = (m_bitoffset+nbits)&7;
			return;
		} else {
			nbits -= bits_avail;
			m_data[pos-1] |= ((value>>nbits)&((1<<bits_avail)-1));
		}
	}
	m_data.resize(pos+((nbits+7)>>3));
	uint8_t* ptr = &m_data[pos];
	while (true) {
		if (nbits <= 8) {
			*ptr = (value<<(8-nbits));
			m_bitoffset = nbits&7;
			return;
		} else {
			nbits -= 8;
			*ptr++ = (value>>nbits);
		}
	}
}

#endif

