/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef REPLACEMENTIMAGE_H_
#define REPLACEMENTIMAGE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include <string>
#include <vector>
#include <utility>

class ReplacementImage
{
public:
	ReplacementImage(int width, int height, bool draw_border, const unsigned char *border_color);
	std::pair<const uint8_t*, const uint8_t*> getBinaryData() const;
	const std::string& getContentType() const { return CONTENT_TYPE; }
private:
	typedef uint16_t LZWCode;
	typedef std::vector<LZWCode> LZWCodeStream;
	static std::string makeScreenDescriptor(int width, int height);
	static std::string makeImageDescriptor(int xpos, int ypos, int width, int height);
	static std::string makeLocalColorTable(const unsigned char *border_color);
	static LZWCodeStream makeLZWCodeStream(LZWCodeStream::size_type data_size, unsigned char color_idx);
	static std::string packLZWCodeStream(const LZWCodeStream& stream);
	static std::string formatLZWByteStream(const std::string& stream);
	static bool isCodeInTable(LZWCodeStream::size_type seqlen, LZWCodeStream::size_type tablepos);
	static LZWCode getCodeFor(LZWCodeStream::size_type seqlen, unsigned char color_idx);
	static const std::string CONTENT_TYPE;
	static const uint8_t HEADER[6];
	static const uint8_t GRAPHIC_CONTROL_EXTENSION[8];
	static const LZWCodeStream::size_type TABLE_SIZE;
	static const LZWCodeStream::size_type FIRST_AVAILABLE_CODE;
	static const LZWCode CLEAR_CODE;
	static const LZWCode EOI_CODE;
	std::string m_data;
};

#endif

