/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "responsefilterchain.h"
#include "httpheaderelement.h"
#include <list>
#include <string>

ResponseFilterChain::ResponseFilterChain(const AbstractResponseFilter::NextFilterProvider& provider)
:	m_terminator(this),
	m_pFirstFilter(&m_terminator),
	m_isSealed(false),
	m_isFinished(false),
	m_userNextFilterProvider(provider),
	m_nextFilterProvider(SigC::slot(*this, &ResponseFilterChain::handleNextFilterRequest))
{
}

ResponseFilterChain::~ResponseFilterChain()
{
	m_pFirstFilter->destroy();
}

AbstractResponseFilter*
ResponseFilterChain::handleNextFilterRequest(HttpResponseMetadata* metadata)
{
	AbstractResponseFilter* next_filter = m_userNextFilterProvider(metadata);
	if (next_filter) {
		return next_filter;
	} else {
		m_isSealed = true;
		m_onMetadataSignal(metadata);
		return &m_terminator;
	}
}

bool
ResponseFilterChain::isBodyEncoded(const HttpResponseMetadata& metadata)
{
	const HttpHeader *teh = metadata.headers().getHeaderPtr("Transfer-Encoding");
	if (teh && !teh->getValue().empty()) {
		return true;
	}
	const HttpHeader *ceh = metadata.headers().getHeaderPtr("Content-Encoding");
	if (ceh && !ceh->getValue().empty()) {
		return true;
	}
	return false;
}

bool
ResponseFilterChain::isEncodingAccepted(const HttpResponseMetadata& metadata, const AcceptEncoding& aenc)
{
	const HttpHeader *teh = metadata.headers().getHeaderPtr("Transfer-Encoding");
	if (teh) {
		const std::list<HttpHeaderElement>& elements = teh->elements().getElements();
		std::list<HttpHeaderElement>::const_iterator iter = elements.begin();
		for (; iter != elements.end(); ++iter) {
			if (!aenc.isAccepted(iter->getLowercaseName())) {
				return false;
			}
		}
	}
	const HttpHeader* ceh = metadata.headers().getHeaderPtr("Content-Encoding");
	if (ceh) {
		const std::list<HttpHeaderElement>& elements = ceh->elements().getElements();
		std::list<HttpHeaderElement>::const_iterator iter = elements.begin();
		for (; iter != elements.end(); ++iter) {
			if (!aenc.isAccepted(iter->getLowercaseName())) {
				return false;
			}
		}
	}
	return true;
}

void
ResponseFilterChain::Terminator::consumeDataChunk(const uint8_t* begin, const uint8_t* end)
{
	m_pChain->m_onDataSignal(begin, end);
}

void
ResponseFilterChain::Terminator::consumeEOF()
{
	m_pChain->m_isFinished = true;
	m_pChain->m_onEofSignal();
}

