/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef RESPONSEFILTERCHAIN_H_
#define RESPONSEFILTERCHAIN_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "abstractresponsefilter.h"
#include "httpresponsemetadata.h"
#include "acceptencoding.h"
#include <sigc++/sigc++.h>

class ResponseFilterChain : public SigC::Object
{
public:
	ResponseFilterChain(const AbstractResponseFilter::NextFilterProvider& provider);
	virtual ~ResponseFilterChain();
private:
	ResponseFilterChain(const ResponseFilterChain&);            //
	ResponseFilterChain& operator=(const ResponseFilterChain&); // not implemeted
public:
	bool isSealed() const { return m_isSealed; }
	bool isFinished() const { return m_isFinished; }
	void consumeMetadata(HttpResponseMetadata* metadata) {
		m_pFirstFilter = handleNextFilterRequest(metadata);
	}
	void consumeDataChunk(const uint8_t* begin, const uint8_t* end) {
		m_pFirstFilter->consumeDataChunk(begin, end);
	}
	void consumeEOF() {
		m_pFirstFilter->consumeEOF();
	}
	SigC::Signal1<void, HttpResponseMetadata*>& onMetadataOutput() {
		return m_onMetadataSignal;
	}
	SigC::Signal2<void, const uint8_t*, const uint8_t*>& onDataOutput() {
		return m_onDataSignal;
	}
	SigC::Signal0<void>& onEOF() {
		return m_onEofSignal;
	}
	AbstractResponseFilter::NextFilterProvider* getNextFilterProvider() { return &m_nextFilterProvider; }
	static bool isBodyEncoded(const HttpResponseMetadata& metadata);
	static bool isEncodingAccepted(const HttpResponseMetadata& metadata, const AcceptEncoding& aenc);
private:
	class Terminator;
	friend class ResponseFilterChain::Terminator;
	struct Terminator : public AbstractResponseFilter
	{
		ResponseFilterChain* m_pChain;
		Terminator(ResponseFilterChain* chain) : m_pChain(chain) {}
		virtual void consumeDataChunk(const uint8_t* begin, const uint8_t* end);
		virtual void consumeEOF();
		virtual void destroy() {}
	};
private:
	AbstractResponseFilter* handleNextFilterRequest(HttpResponseMetadata* metadata);
private:
	Terminator m_terminator;
	AbstractResponseFilter* m_pFirstFilter;
	bool m_isSealed;
	bool m_isFinished;
	AbstractResponseFilter::NextFilterProvider m_userNextFilterProvider;
	AbstractResponseFilter::NextFilterProvider m_nextFilterProvider;
	SigC::Signal1<void, HttpResponseMetadata*> m_onMetadataSignal;
	SigC::Signal2<void, const uint8_t*, const uint8_t*> m_onDataSignal;
	SigC::Signal0<void> m_onEofSignal;
};

#endif

