/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SERVERINFO_H_
#define SERVERINFO_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpversion.h"

// This class represents a server or a proxy
class ServerInfo
{
public:
	enum PipeliningCapability { P_UNKNOWN, P_TESTING, P_OK, P_NOT_OK };
	ServerInfo();
	const HttpVersion& getProtocolCompability() const;
	void setProtocolCompability(const HttpVersion& version);
	PipeliningCapability getPipeliningCapability() const;
	void setPipeliningCapability(PipeliningCapability pcap);
	int getReceivedRequestsCount() const { return m_received_requests; }
	int getServedRequestsCount() const { return m_served_requests; }
	void registerRequest() { ++m_received_requests; }
	void registerResponse(int code);
	void reset();
private:
	HttpVersion m_protocol_compability;
	PipeliningCapability m_pipelining_capability;
	int m_received_requests;
	int m_served_requests;
};

inline
ServerInfo::ServerInfo()
:	m_protocol_compability(HttpVersion::HTTP_0_9), m_pipelining_capability(P_UNKNOWN),
	m_received_requests(0), m_served_requests(0)
{}

inline const HttpVersion&
ServerInfo::getProtocolCompability() const
{
	return m_protocol_compability;
}

inline void
ServerInfo::setProtocolCompability(const HttpVersion& proto_version)
{
	m_protocol_compability = proto_version;
}

inline ServerInfo::PipeliningCapability
ServerInfo::getPipeliningCapability() const
{
	return m_pipelining_capability;
}

inline void
ServerInfo::setPipeliningCapability(PipeliningCapability pcap)
{
	m_pipelining_capability = pcap;
}

inline void
ServerInfo::registerResponse(int code)
{
	if (code >= 100 && code < 200) {
		return;
	}
	++m_served_requests;
}

inline void
ServerInfo::reset()
{
	m_protocol_compability = HttpVersion::HTTP_0_9;
	m_pipelining_capability = P_UNKNOWN;
	m_received_requests = 0;
	m_served_requests = 0;
}

#endif

