/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SERVERRESPONSESTREAM_H_
#define SERVERRESPONSESTREAM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types.h"
#include "httpstream.h"
#include "httpstreamwriter.h"
#include "abstractresponsefilter.h"
#include "responsefilterchain.h"
#include "httpclientrequest.h"
#include "namevaluepair.h"
#include "intrusiveptr.h"
#include <sigc++/sigc++.h>
#include <memory>
#include <deque>

class BFilter;

class ServerResponseStream : public HttpStream, protected HttpStreamWriter::Receiver, public SigC::Object
{
public:
	ServerResponseStream(BFilter* owner);
private:
	ServerResponseStream(const ServerResponseStream&);            //
	ServerResponseStream& operator=(const ServerResponseStream&); // not implemented
public:
	bool isForwardingInProgress() { return m_ptrStreamWriter.get(); }
	void processSubstitutionRequest(const HttpClientRequest *request); // to be called from ClientRequestStream and locally
protected:
	/* Inherited from HttpStream */
	virtual bool isRequestStream() const { return false; }
	virtual bool isResponseOnHeadRequest() const;
	virtual void eventError(int ecode, const char *info=0);
	virtual void eventGotMetadata(HttpRequestMetadata& metadata) {}
	virtual void eventGotMetadata(HttpResponseMetadata& metadata);
	virtual void eventGotContentPart(const uint8_t* begin, const uint8_t* end);
	virtual void eventGotFooters(const HttpHeadersCollection& footers) {}
	virtual void eventMessageEnds(bool connection_needs_closing);
	
	/* Inherited from HttpStreamWriter::Receiver */
	virtual void moreData(const uint8_t* begin, const uint8_t* end);
	virtual void done(bool conn_close);
private:
	AbstractResponseFilter* provideNextFilter(HttpResponseMetadata* metadata);
	AbstractResponseFilter* tryBlackHoleFilter(HttpResponseMetadata* metadata);
	AbstractResponseFilter* tryHtmlFilter(HttpResponseMetadata* metadata);
	AbstractResponseFilter* tryImageFilter(HttpResponseMetadata* metadata);
	AbstractResponseFilter* tryFlashFilter(HttpResponseMetadata* metadata);
	AbstractResponseFilter* tryDecompressorFilter(HttpResponseMetadata*);
	AbstractResponseFilter* tryCompressorFilter(HttpResponseMetadata*);
	void processFilteredMetadata(HttpResponseMetadata* metadata);
	void processFilteredData(const uint8_t* begin, const uint8_t* end);
	void processFilterEOF();
	bool isFilteredResponse(const HttpResponseMetadata& metadata);
	void handleRedirectWhileAnalyzingImage(HttpResponseMetadata& metadata);
	void handleRedirectWhileAnalyzingFlash(HttpResponseMetadata& metadata);
	void prepareResponseForForwarding(HttpResponseMetadata& metadata);
	void performImageSubstitution(const HttpClientRequest *request);
	void performFlashSubstitution(const HttpClientRequest *request);
	void performErrorSubstitution(const HttpClientRequest *request);
	
	BFilter* m_pOwner;
	IntrusivePtr<HttpClientRequest> m_ptrActiveRequest;
	std::auto_ptr<ResponseFilterChain> m_ptrFilterChain;
	std::auto_ptr<HttpStreamWriter> m_ptrStreamWriter;
	AbstractResponseFilter::NextFilterProvider m_nextFilterProvider;
	std::deque<AbstractResponseFilter::NextFilterProvider> m_filterTryList;
	bool m_is1xxResponse;
};

#endif

