/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "stringtokenizer.h"

StringTokenizer::StringTokenizer(const std::string& str, const std::string& delim, bool return_delims)
:	m_str(str), m_delim(delim), m_return_delims(return_delims), m_cur_pos(0)
{}

bool
StringTokenizer::hasMoreTokens()
{
	if (!m_next_token.empty()) {
		return true;
	} else if (m_cur_pos == std::string::npos || m_cur_pos == m_str.length()) {
		return false;
	} else {
		if (m_return_delims) {
			std::size_t delim_pos = m_str.find_first_of(m_delim, m_cur_pos);
			if (delim_pos == m_cur_pos) {
				std::size_t ndelim_pos = m_str.find_first_not_of(m_delim, m_cur_pos);
				if (ndelim_pos == std::string::npos) {
					ndelim_pos = m_str.length();
				}
				m_next_token = m_str.substr(m_cur_pos, ndelim_pos-m_cur_pos);
				m_cur_pos = ndelim_pos;
				return true;
			} else if (delim_pos == std::string::npos) {
				delim_pos = m_str.length();
			}
			m_next_token = m_str.substr(m_cur_pos, delim_pos-m_cur_pos);
			m_cur_pos = delim_pos;
			return true;
		} else {
			m_cur_pos = m_str.find_first_not_of(m_delim, m_cur_pos);
			if (m_cur_pos == std::string::npos) {
				return false;
			}
			std::string::size_type next_pos = m_str.find_first_of(m_delim, m_cur_pos);
			if (next_pos == std::string::npos) {
				next_pos = m_str.length();
			}
			m_next_token = m_str.substr(m_cur_pos, next_pos-m_cur_pos);
			m_cur_pos = next_pos;
			return true;
		}
	}
}

std::string
StringTokenizer::nextToken()
{
	hasMoreTokens(); // prepare the next token
	std::string tmp = m_next_token;
	m_next_token.erase();
	return tmp;
}

