/*
    BFilter - a web proxy which removes banner ads
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef TSELECT_H_
#define TSELECT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <map>
#include <set>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif

class TSelect
{
public:
	TSelect() : m_time_passed(0, 0) {}
	
	int addTimer(struct timeval& tv);
	
	void removeTimer(int id);
	
	void resetTimer(int id);
	
	int select(int n, fd_set *rfds, fd_set *wfds, fd_set *efds);
	
	int selectNoRetry(int n, fd_set *rfds, fd_set *wfds, fd_set *efds);
	
	const std::set<int>& getTriggeredTimers() { return m_triggered_timers; }
private:
	class Timeval
	{
	public:
		Timeval() : m_sec(0), m_usec(0) {}
		
		Timeval(long sec, long usec) : m_sec(sec), m_usec(usec) { normalize(); }
		
		long getSec() const { return m_sec; }
		
		long getUsec() const { return m_usec; }
		
		bool operator<(const Timeval& tv) const;
		
		Timeval operator+(const Timeval& tv) const;
		
		Timeval operator-(const Timeval& tv) const;
	private:
		void normalize();
		
		long m_sec;
		long m_usec;
	};
private:
	typedef std::multimap<Timeval, int> TimerMap;
	TimerMap m_timers;
	std::map<int, Timeval> m_timers_by_id;
	Timeval m_time_passed;
	std::set<int> m_triggered_timers;
};

#endif

